"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Floor = exports.CameraThing = exports.GRAPH = exports.VT52 = exports.Capsule = exports.Point = exports.Torus = exports.Cone = exports.Cylinder = exports.Sphere = exports.Cube = exports.ThingCommon = exports.dynamicY = exports.dynamicX = void 0;
const babyengine_1 = require("../middle/babyengine");
const babything_1 = require("../frontend/babything");
const babylonjs_1 = require("babylonjs");
const babylonjs_2 = require("babylonjs");
const babylonjs_materials_1 = require("babylonjs-materials");
const consoleRed = 'background-color:red;color:white;';
const consoleYellow = 'background-color:yellow;color:black;';
const consoleBlue = 'background-color:blue;color:white;';
exports.dynamicX = 1048; // the slightly unusual 1048 resolution enables the VT52 mode  (1024 plus one more)
exports.dynamicY = 1048;
// need to export this, because it is the 'type' for Ball, Box, etc
class ThingCommon {
    constructor(cmd, handle, scene) {
        this.params = {};
        this.scene = scene;
        this.handle = handle;
        // we want this.handle.thing to be set to this, but do it at the
        // lowest subclass so we get the right instanceof value
    }
    standardMeshSetup() {
        if (this.handle.babylonMesh instanceof babylonjs_1.Camera) { // typeguard
            console.error('MeshSetup only for mesh');
        }
        else {
            const material = new babylonjs_1.StandardMaterial('', this.scene);
            this.handle.type = 'mesh'; // a VT52 is a mesh too
            this.handle.babylonMesh.scaling = new babylonjs_1.Vector3(.99, .99, .99); // just a fraction less
            this.handle.babylonMesh.material = material;
            this.handle.material = material; // because we can't always get to handle.babylonMesh.material.diffuseColor...
            // the slightly unusual 1048 resolution enables the VT52 mode  (1024 plus one more)
            this.handle.dynamicTexture = new babylonjs_1.DynamicTexture("", { width: exports.dynamicX, height: exports.dynamicY }, babyengine_1.BabyEngine.babylonShapes.scene, true);
            this.handle.mass = 0; // a ghost unless we find out otherwise
            this.handle.restitution = .8;
        }
    }
    /** if Ammo tells us a collision happened, this cleans up and notifies the parties */
    onCollisionHelper(a, b) {
        if (a === b) { // sometimes we seem to collide with ourselves ?!?
            // console.log('%cA and B are === - we collided with ourself',consoleRed)
            return;
        }
        // lookup the uuid for a and b, and send BOTH a message
        let aMesh = null;
        let aMeshHandle = null;
        let bMesh = null;
        let bMeshHandle = null;
        if (aMeshHandle.babylonMesh instanceof babylonjs_1.Mesh &&
            bMeshHandle.babylonMesh instanceof babylonjs_1.Mesh) { // typeguard
            babyengine_1.BabyEngine.shapeMap.forEach((mesh) => {
                if (mesh.babylonMesh instanceof babylonjs_1.Mesh) { // typeguard
                    // we only allow a shape to collide once every 100ms
                    // console.log(mesh.babylonMesh.physicsImpostor)
                    if (mesh.babyShape instanceof babything_1.BabyShape) { // not a  BabyButton 
                        if (typeof mesh.babylonMesh.physicsImpostor === 'object' &&
                            mesh.lastCollisionTime < (Date.now() - 10)) {
                            if (mesh.babylonMesh.physicsImpostor === a) {
                                aMesh = mesh.babyShape; // get the babyShape for this mesh
                                aMeshHandle = mesh;
                            }
                            if (mesh.babylonMesh.physicsImpostor === b) {
                                bMesh = mesh.babyShape;
                                bMeshHandle = mesh;
                            }
                        }
                    }
                }
            });
        }
        // ok, did we get TWO shapes?  (many reasons we might not)
        if (aMesh !== null && bMesh !== null) {
            // signal both that they hit each other... likely only one is listening
            babyengine_1.BabyEngine.notifyObservers(`collide.${aMesh.uuid}`, bMesh);
            babyengine_1.BabyEngine.notifyObservers(`collide.${bMesh.uuid}`, aMesh);
            // this is a small debounce.  neither will hit for 100th of a second
            aMeshHandle.lastCollisionTime = Date.now();
            bMeshHandle.lastCollisionTime = Date.now();
        }
    }
    /** setup collider for a shape */
    setupCollider() {
        // TODO:  something wrong here, not sure what.  come back to it, comment out for now
        // console.log('%csetupCollider',consoleRed,this.handle)
        if (this.handle.babylonMesh instanceof babylonjs_1.Mesh) { // typeguard
            // set up collider
            // this.handle.babylonMesh.physicsImpostor = new PhysicsImpostor(this.handle.babylonMesh, PhysicsImpostor[this.impostorString], { mass: this.handle.mass, restitution: this.handle.restitution }, this.scene)
            // BabyEngine.impostors.push(this.handle.babylonMesh.physicsImpostor)
        }
    }
    /** dispose of a mesh (and its physics impostor) */
    dispose() {
        this.handle.tweenArray = [];
        if (this.handle.babylonMesh instanceof babylonjs_1.Mesh) { // typeguard
            this.handle.babylonMesh.material.dispose(); // do I have to check it exists?
            // yank the impostor out of our list of impostors, if it is there
            this.disposeImpostor();
            // and finally dispose of the mesh itself
            this.handle.babylonMesh.dispose();
        }
        // TODO: get rid of the collide and click observables
    }
    /** create physics impostor for this shape */
    createPhysicsImpostor() {
        if (this.handle.babylonMesh instanceof babylonjs_1.Mesh) { // typeguard
            // console.log('checking...',
            //     // this.handle.babylonMesh,
            //     PhysicsImpostor[this.impostorString]
            // )
            let mass = 0; // default for ghost and solid
            if (this.handle.collider == 'active')
                mass = this.handle.mass;
            this.handle.babylonMesh.physicsImpostor =
                new babylonjs_2.PhysicsImpostor(this.handle.babylonMesh, babylonjs_2.PhysicsImpostor[this.impostorString], { mass: mass, restitution: this.handle.restitution }, this.scene);
            babyengine_1.BabyEngine.impostors.push(this.handle.babylonMesh.physicsImpostor);
        }
    }
    /** dispose of the physics impostor belonging to a mesh */
    disposeImpostor() {
        if (this.handle.babylonMesh instanceof babylonjs_1.Mesh) { // typeguard
            // yank the impostor out of our list of impostors, if it is there
            const temp = babyengine_1.BabyEngine.impostors.length; // assert that we will remove one element
            babyengine_1.BabyEngine.impostors.splice(babyengine_1.BabyEngine.impostors.findIndex(e => {
                if (this.handle.babylonMesh instanceof babylonjs_1.Mesh) { // typeguard
                    e === this.handle.babylonMesh.physicsImpostor;
                }
            }), 1);
            this.handle.babylonMesh.physicsImpostor.dispose(); // do I have to check?
            this.handle.babylonMesh.physicsImpostor = null; // make sure
            console.assert(temp == babyengine_1.BabyEngine.impostors.length + 1, `started with ${temp} impostors, removed one, now have ${babyengine_1.BabyEngine.impostors.length} ?!?`);
        }
        else
            console.error('Physics Impostor only on Mesh');
    }
    /** can't resize an impostor, so drop it and recreate  */
    recreatePhysicsImpostor() {
        if (this.handle.babylonMesh instanceof babylonjs_1.Mesh) { // typeguard
            // first step - remove the old impostor from the global list of impostors, and dispose of it
            this.disposeImpostor();
            // second step - recreate it with the new dimensions
            // TODO decide whether box or ball...
            switch (this.handle.model) {
                case 'cube':
                    this.handle.babylonMesh.physicsImpostor = new babylonjs_2.PhysicsImpostor(this.handle.babylonMesh, babylonjs_2.PhysicsImpostor.BoxImpostor, { mass: this.handle.mass, restitution: this.handle.restitution }, this.scene);
                    if (this.scene.debugLayer.isVisible()) {
                        babyengine_1.BabyEngine.physicsViewer.showImpostor(this.handle.babylonMesh.physicsImpostor);
                    }
                    break;
                default:
                    console.error('should not get here, do not have a handler for ' + this.handle.model);
            }
            // has to work for meshs, cameras, and lights, but only shapes has scaling
            // push the new resized impostor back on the global list of impostors
            babyengine_1.BabyEngine.impostors.push(this.handle.babylonMesh.physicsImpostor);
            this.handle.babylonMesh.physicsImpostor.registerOnPhysicsCollide(babyengine_1.BabyEngine.impostors, (a, b) => {
                this.onCollisionHelper(a, b);
            });
        }
        else
            console.error('Physics Impostor only on Mesh');
    }
    showDebugImpostor() {
    }
    setShapeMass() {
    }
}
exports.ThingCommon = ThingCommon;
class Cube extends ThingCommon {
    constructor(cmd, handle, scene) {
        super(cmd, handle, scene);
        this.handle.babylonMesh = babylonjs_1.MeshBuilder.CreateBox(this.handle.uniqueName, this.params, this.scene);
        this.standardMeshSetup();
        this.handle.mass = 1;
        this.impostorString = 'BoxImpostor';
        this.createPhysicsImpostor();
    }
}
exports.Cube = Cube;
class Sphere extends ThingCommon {
    constructor(cmd, handle, scene) {
        super(cmd, handle, scene);
        this.handle.babylonMesh = babylonjs_1.MeshBuilder.CreateSphere(handle.uniqueName, this.params, this.scene);
        this.standardMeshSetup();
        this.handle.mass = (4 / 3) * Math.PI; // times R^3, which is just 1
        this.impostorString = 'SphereImpostor';
        this.createPhysicsImpostor();
    }
}
exports.Sphere = Sphere;
// a can and a cone are really the same thing - a cylinder
class Cylinder extends ThingCommon {
    constructor(cmd, handle, scene) {
        super(cmd, handle, scene);
        this.handle.babylonMesh = babylonjs_1.MeshBuilder.CreateCylinder(handle.uniqueName, this.params, this.scene);
        this.standardMeshSetup();
        this.handle.mass = (4 / 3) * Math.PI; // times R^3, which is just 1
        this.impostorString = 'SphereImpostor';
        this.createPhysicsImpostor();
    }
}
exports.Cylinder = Cylinder;
class Cone extends ThingCommon {
    constructor(cmd, handle, scene) {
        super(cmd, handle, scene);
        this.params = { diameterTop: 0 };
        this.handle.babylonMesh = babylonjs_1.MeshBuilder.CreateCylinder(handle.uniqueName, this.params, this.scene);
        this.standardMeshSetup();
        this.handle.mass = (4 / 3) * Math.PI; // times R^3, which is just 1
        this.impostorString = 'SphereImpostor';
        this.createPhysicsImpostor();
    }
}
exports.Cone = Cone;
class Torus extends ThingCommon {
    constructor(cmd, handle, scene) {
        super(cmd, handle, scene);
        this.handle.babylonMesh = babylonjs_1.MeshBuilder.CreateTorus(handle.uniqueName, this.params, this.scene);
        this.standardMeshSetup();
        this.handle.mass = (4 / 3) * Math.PI; // times R^3, which is just 1
        this.impostorString = 'SphereImpostor';
        this.createPhysicsImpostor();
    }
}
exports.Torus = Torus;
class Point extends ThingCommon {
    constructor(cmd, handle, scene) {
        super(cmd, handle, scene);
        this.handle.babylonMesh = babylonjs_1.MeshBuilder.CreateSphere(handle.uniqueName, this.params, this.scene);
        this.standardMeshSetup();
        this.handle.mass = (4 / 3) * Math.PI; // times R^3, which is just 1
        this.impostorString = 'SphereImpostor';
        this.createPhysicsImpostor();
    }
}
exports.Point = Point;
// // we create TWO disks, and glue them together (otherwise only opaque one side)
// export class Disc extends ThingCommon implements Thing {
//     constructor(cmd: gameCmd, handle: MeshHandle, scene: Scene) {
//         super(cmd, handle, scene)
//         let params = {radius: 0.5, tessellation: 64}
//         this.handle.babylonMesh = MeshBuilder.CreateDisc(handle.uniqueName, this.params, this.scene)
//         this.standardMeshSetup()
//         let temp = MeshBuilder.CreateDisc(handle.uniqueName, this.params, this.scene)
//         temp.rotate(new Vector3(1,0,0),Math.PI)
//         temp.parent = this.handle.babylonMesh
//         this.handle.mass = (4 / 3) * Math.PI // times R^3, which is just 1
//         this.impostorString = 'SphereImpostor'
//     }
// }
class Capsule extends ThingCommon {
    constructor(cmd, handle, scene) {
        super(cmd, handle, scene);
        // bug in BabylonJS?  defaults not defined?
        let params = { subdivisions: 2, tessellation: 16, height: 1, radius: 0.25, capSubdivisions: 6 };
        this.handle.babylonMesh = babylonjs_1.MeshBuilder.CreateCapsule(handle.uniqueName, params, this.scene);
        this.standardMeshSetup();
        this.handle.mass = (4 / 3) * Math.PI; // times R^3, which is just 1
        this.impostorString = 'SphereImpostor';
        this.createPhysicsImpostor();
    }
}
exports.Capsule = Capsule;
// some defines for the VT52
const VT52pixelX = 13;
const VT52pixelY = 31;
const VT52rows = 34; // not 24 like the old terminal
const VT52cols = 80;
const VT52vOffset = 20; // correct for characters (not for squares)
const VT52font = "20px Courier";
const scalingX = 12; // this gives a roughly square look
const scalingY = 16;
class VT52 extends ThingCommon {
    constructor(cmd, handle, scene) {
        super(cmd, handle, scene);
        this.printBuffer = [];
        this.cursorX = 0;
        this.cursorY = 0;
        this.vt52print = () => { return; }; // used to store a print() resolve()
        this.vt52input = (s) => { return s; };
        // used to store an input() resolve
        this.inputBuffer = '';
        this.cursorSuppress = false; // suppress flashingcursor (for graphics?)
        this.halfSeconds = 0;
        this.cursorOn = true; // used for state of flashing cursor
        this.handle.babylonMesh = babylonjs_1.MeshBuilder.CreateBox(handle.uniqueName, this.params, this.scene);
        this.standardMeshSetup();
        this.handle.babylonMesh.scaling = new babylonjs_1.Vector3(scalingX, 0.1, scalingY);
        this.handle.babylonMesh.position.y -= 0.5; // move to match cube footprint
        // // add axes so i can see what's going on
        // let cmd2: gameCmd = { action: 'addAxes', uuid: this.handle.uuid }
        // BabyEngine.babylonShapes.addAxes(cmd2, this.handle)
        this.handle.babylonMesh.rotate(new babylonjs_1.Vector3(0, 1, 0), -Math.PI / 2);
        // floor has mass 0, always passive (set up Impostor, no callback)
        this.handle.mass = 0;
        this.handle.restitution = .3;
        this.handle.collider = 'solid';
        this.impostorString = 'BoxImpostor';
        this.createPhysicsImpostor();
        if (this.handle.dynamicTexture === undefined) {
            this.handle.material.diffuseTexture = handle.dynamicTexture;
        }
        this.handle.material.diffuseTexture = handle.dynamicTexture;
        // this.handle.material.diffuseTexture = this.handle.dynamicTexture
        this.ctx = this.handle.dynamicTexture.getContext();
        // // reposition the camera to look at the VT52 screen
        let cameraHandle = babyengine_1.BabyEngine.babylonShapes.activeCameraHandle;
        // can't set position of most cameras
        if (cameraHandle.babylonMesh instanceof babylonjs_1.ArcRotateCamera) { // typeguard, of course it is
            // cameraHandle.babylonMesh.setPosition(new Vector3(0, 0, 0))
            cameraHandle.babylonMesh.setPosition(new babylonjs_1.Vector3(0, 15, 0));
        }
        this.displayCharBuffer = Array(VT52rows * VT52cols).fill(32); // space chars
        this.displayColorBuffer = Array(VT52rows * VT52cols).fill('green');
        // finally, add a 'tick' observer to keep this vt52 running in the background
        babyengine_1.BabyEngine.addObserver('tick', (data) => {
            this.printDaemon();
            this.cursorDaemon();
        });
        babyengine_1.BabyEngine.addObserver('keydown', (data) => {
            this.input(data);
        });
        let camera = babyengine_1.BabyEngine.babylonShapes.activeCameraHandle.babylonMesh;
        if (camera instanceof babylonjs_1.Camera)
            camera.detachControl();
    }
    setCursor(x, y) {
        this.cursorX = x;
        this.cursorY = y;
    }
    cursorDaemon() {
        if (this.cursorSuppress) // maybe we don't want cursor at all
            return;
        let halfSeconds = Math.floor(new Date().getTime() / 500); // half-seconds since Jan 1, 1970
        if (halfSeconds !== this.halfSeconds) {
            // time to twiddle the cursor
            // if (this.cursorOn){
            //     this.drawScreen()  // this will erase it 
            // } else {
            let charString = '_';
            this.ctx.fillStyle = this.cursorOn ? 'white' : 'black';
            this.ctx.fillText(charString, this.cursorX * VT52pixelX, this.cursorY * VT52pixelY + VT52vOffset);
            this.handle.dynamicTexture.update();
            // }
            // set up for next cycle
            this.cursorOn = !this.cursorOn;
            this.halfSeconds = halfSeconds; // don't want to hear from you for a while
        }
    }
    printDaemon() {
        if (this.printBuffer.length > 0) {
            let p = this.printBuffer.shift();
            this.printChar(p.char, p.color);
            // refresh the VT52 screen buffer
            this.drawScreen();
            // and update the texture
            this.handle.dynamicTexture.update();
        }
        else {
            // there may be a Promise waiting for this to finish
            // if resolve() is not undefined, then fire it and set it undefined
            this.vt52print();
        }
    }
    print(text, color) {
        // process the string into the printBuffer queue, that's all
        text.split('').forEach((char) => this.printBuffer.push({ char: char.charCodeAt(0), color: color }));
    }
    // full refresh of VT52 screen.
    drawScreen() {
        this.ctx.font = VT52font;
        this.ctx.clearRect(0, 0, exports.dynamicX, exports.dynamicY); // transparent black
        this.ctx.beginPath(); // reset after clear
        for (let j = 0; j < VT52rows; j++) {
            for (let i = 0; i < VT52cols; i++) {
                let charCode = this.displayCharBuffer[j * VT52cols + i];
                let charColor = this.displayColorBuffer[j * VT52cols + i];
                // if (charCode === 0) {
                // a null shows as a small blue rectangle
                // this.ctx.fillStyle = 'blue'
                // this.ctx.fillRect(i * VT52pixelX + 4, j * VT52pixelY + 6, 2, 2)
                // } else {
                // console.log('charCode', j, i, charCode)
                let charString = String.fromCharCode(charCode);
                this.ctx.fillStyle = charColor;
                this.ctx.fillText(charString, i * VT52pixelX, j * VT52pixelY + VT52vOffset);
                // }
            }
        }
    }
    printChar(charCode, charColor) {
        console.assert(this.cursorX < VT52cols, `cursorX should always be less than ${VT52cols}`);
        if (charCode === 10) { // force CRLF
            this.printCRLF();
        }
        else {
            this.displayCharBuffer[VT52cols * this.cursorY + this.cursorX] = charCode;
            this.displayColorBuffer[VT52cols * this.cursorY + this.cursorX] = charColor;
            this.cursorX += 1; // horizontal cursor position
            if (this.cursorX >= VT52cols) {
                this.printCRLF();
                this.cursorX = 0;
            }
        }
    }
    printCRLF() {
        // console.log(`printCRLF at ${this.cursorX},${this.cursorY}`)
        if (this.cursorY === (VT52rows - 1)) {
            // we are on the bottom line, so scroll up
            for (let i = VT52cols; i < this.displayCharBuffer.length; i++) { // starting at 80 (second line)
                this.displayCharBuffer[i - VT52cols] = this.displayCharBuffer[i];
                this.displayColorBuffer[i - VT52cols] = this.displayColorBuffer[i];
                // this.displayCharBuffer[i] = 32          
                // this.displayColorBuffer[i] = 'green'
            }
            this.cursorX = 0; // cursorY remains the bottom row
            // clear the bottom line
            let firstCol = ((VT52rows - 1) * VT52cols) - 1;
            for (let i = 0; i < VT52cols; i++) {
                // console.log ('clearing VT52',i+firstCol)
                this.displayCharBuffer[i + firstCol] = 32; // space char
                this.displayColorBuffer[i + firstCol] = 'green';
            }
        }
        else {
            // just drop the cursor to the next line
            this.cursorY += 1;
            this.cursorX = 0;
        }
    }
    // async vinput(prompt: string = ''): Promise<string> {
    //     console.log('arrived in input')
    //     if (prompt !== undefined)
    //         this.print(prompt, 'white')
    //     this.vt52input = (s: string) => { return (s) }
    //     return this.vt52input(this.inputBuffer)
    // }
    input(data) {
        let code = data['keyCode'];
        console.log('in VT52.input()', code, data);
        // before anything, erase the cursor
        let charString = '_';
        this.ctx.fillStyle = 'black';
        this.ctx.fillText(charString, this.cursorX * VT52pixelX, this.cursorY * VT52pixelY + VT52vOffset);
        // seem to get codes up to 256  (eg: '?' is 191)
        if (code > 128)
            code -= 128;
        // now print a character if possible
        if ((code % 128) >= 32 && code <= 126) { // the printables
            this.inputBuffer += data['key']; //String.fromCharCode(code)
            this.printChar(data['key'].toString().charCodeAt(0), 'white');
        }
        else {
            switch (code) {
                case 13: // Enter
                    // if resolve() is not undefined, then fire it and set it undefined
                    this.vt52input(this.inputBuffer); // fire it off with our buffer
                    this.inputBuffer = '';
                    this.printCRLF();
                    break;
                case 8: // BackSpace
                    // if any of the stuff on the screen is being collected
                    if (this.inputBuffer.length > 0) {
                        this.inputBuffer = this.inputBuffer.substr(0, this.inputBuffer.length - 1);
                        // if we are not at the start of a line (otherwise too bad)
                        if (this.cursorX > 0) {
                            this.cursorX -= 1;
                            // update the buffer, char will disappear at next drawScreen()
                            this.displayCharBuffer[VT52cols * this.cursorY + this.cursorX] = 32; // a space
                            // color doesn't matter, it's a space
                        }
                    }
                    break;
            }
        }
        this.drawScreen();
        this.handle.dynamicTexture.update();
        return;
    }
}
exports.VT52 = VT52;
class GRAPH extends ThingCommon {
    // a graph is just a thin box on the xy0 plane
    constructor(cmd, handle, scene) {
        super(cmd, handle, scene);
        this.scaleX = cmd.x;
        this.scaleY = cmd.y;
        this.sX = exports.dynamicX / this.scaleX; // note use scaleY for x pixels 
        this.sY = exports.dynamicY / this.scaleY;
        this.zeroX = this.sX * this.scaleX / 2;
        this.zeroY = this.sY * this.scaleY / 2;
        this.handle.babylonMesh = babylonjs_1.MeshBuilder.CreateBox(handle.uniqueName, this.params, this.scene);
        this.standardMeshSetup();
        this.handle.babylonMesh.scaling = new babylonjs_1.Vector3(cmd.x, cmd.y, 0.1);
        // this.handle.babylonMesh.rotate(new Vector3(1, 0, 0), Math.PI /2)
        this.handle.babylonMesh.rotate(new babylonjs_1.Vector3(0, 1, 0), Math.PI);
        // this.handle.babylonMesh.position.z = -0.5 // move to match cube footprint
        // this.handle.babylonMesh.scaling = new Vector3(scalingX, 0.1, scalingY)
        // this.handle.babylonMesh.position.y -= 0.5 // move to match cube footprint
        // this.handle.babylonMesh.rotate(new Vector3(0, 1, 0), -Math.PI / 2)
        // // add axes so i can see what's going on
        // let cmd2: gameCmd = { action: 'addAxes', uuid: this.handle.uuid }
        // BabyEngine.babylonShapes.addAxes(cmd2, this.handle)
        // floor has mass 0, always passive (set up Impostor, no callback)
        this.handle.mass = 0;
        this.handle.restitution = .3;
        this.handle.collider = 'ghost';
        this.impostorString = 'BoxImpostor';
        // this.createPhysicsImpostor()
        if (this.handle.dynamicTexture === undefined) {
            this.handle.material.diffuseTexture = handle.dynamicTexture;
        }
        this.handle.material.diffuseTexture = handle.dynamicTexture;
        // this.handle.material.diffuseTexture = this.handle.dynamicTexture
        this.ctx = this.handle.dynamicTexture.getContext();
        // clear grid to white
        this.ctx.fillStyle = 'white';
        this.ctx.fillRect(0, 0, exports.dynamicX, exports.dynamicY);
        this.handle.dynamicTexture.update();
        // // reposition the camera to look at the VT52 screen
        let cameraHandle = babyengine_1.BabyEngine.babylonShapes.activeCameraHandle;
        // can't set position of most cameras
        if (cameraHandle.babylonMesh instanceof babylonjs_1.ArcRotateCamera) { // typeguard, of course it is
            // cameraHandle.babylonMesh.setPosition(new Vector3(0, 0, 0))
            // cameraHandle.babylonMesh.setPosition(new Vector3(0, -15, 0))
            // cameraHandle.babylonMesh.setMatUp()
        }
        // let camera = BabyEngine.babylonShapes.activeCameraHandle.babylonMesh
        // if (camera instanceof Camera)
        //     camera.detachControl()
        // this.drawGrid(10,10)
    }
    // a graph is just a VT52 terminal with some gridlines on it
    // but we have to be careful.  the VT52 base is exactly scalingX by scalingY units (12x16)
    // with dynamicX by dynamicY pixels across.  we want to orient the lines exactly on
    // the scaling grid
    /** figures out how to draw a nice grid */
    drawGrid(x, y) {
        //TODO this scaleX should not be the same as the graph's SCALEX
        // it represents the gridlines, not the graphsize
        this.scaleX = x;
        this.scaleY = y;
        let minX = this.scaleX / -2;
        let maxX = this.scaleX / 2;
        let minY = this.scaleY / -2;
        let maxY = this.scaleY / 2;
        // the xAxis draws across the Y values
        for (let x = minX; x < maxX; x++) {
            let xLine = (exports.dynamicX / this.scaleX) * (maxX - x); // number of pixels from LEFT
            this.ctx.beginPath();
            this.ctx.lineWidth = (x == 0) ? 3 : 1;
            this.ctx.moveTo(xLine, 0);
            this.ctx.lineTo(xLine, exports.dynamicY);
            this.ctx.stroke(); // render
            if (x !== 0)
                this.textAt(x.toString(), x, 0, 4, -4); // this is drawn up the y axis
        }
        // the yAxis draws across the X values
        for (let y = minY; y < maxY; y++) {
            let yLine = (exports.dynamicY / this.scaleY) * (maxY - y); // number of pixels from TOP
            this.ctx.beginPath();
            this.ctx.lineWidth = (y == 0) ? 3 : 1;
            this.ctx.moveTo(0, yLine);
            this.ctx.lineTo(exports.dynamicX, yLine);
            this.ctx.stroke(); // render
            if (y !== 0)
                this.textAt(y.toString(), 0, y, 4, -4); // this is drawn across X axis
        }
        // finally
        this.handle.dynamicTexture.update();
    }
    /** writes text, perhaps slightly offset right and above, like grid markings */
    textAt(text, x, y, offsetX = 0, offsetY = 0) {
        let draw = this.localXY([x, y]);
        this.ctx.font = '24px serif';
        this.ctx.fillStyle = 'blue';
        // this.ctx.fillText(text, drawX + offsetX, drawY + offsetY)
        this.ctx.fillText(text, draw[0] + offsetX, draw[1] + offsetY);
        // finally
        this.handle.dynamicTexture.update();
    }
    /** x,y is start, rest is a JSON string of array of subsequent points */
    drawLine(markerSymbol, x, y, restOfLine) {
        let rest = JSON.parse(restOfLine);
        console.assert(Array.isArray(rest), "expected an array of [number,number]");
        this.ctx.beginPath();
        this.ctx.lineWidth = 3;
        // let localPoint = this.localXY(x, y)
        this.localMoveTo([x, y]); // y-positive moves UP
        rest.forEach((point) => {
            this.localLineTo(point);
        });
        this.ctx.stroke();
        // now go through the lines again and add the marker symbols
        let lastX = x;
        let lastY = y;
        rest.forEach((point) => {
            let length = Math.sqrt((point[0] - lastX) * (point[0] - lastX) + (point[1] - lastY) * (point[1] - lastY)); //x**2 + y**2
            let unitV = [(point[0] - lastX) / length, (point[1] - lastY) / length];
            let perpendV = [unitV[1], unitV[0]]; // just spin them
            console.log('length,unit,perpendV', [x, y], length, unitV, perpendV);
            if (markerSymbol == 'arrow' || markerSymbol == 'arrowdot') {
                // add a triangle to the end of the line
                // the starting point is always the END point of the line
                this.ctx.beginPath();
                this.localMoveTo(point); // move to end of line segment
                this.localLineTo([point[0] + (unitV[0] * -0.3) + (perpendV[0] * +.12), point[1] + (unitV[1] * -0.3) + (perpendV[1] * -0.12)]); // y-positive moves UP
                this.localLineTo([point[0] + (unitV[0] * -0.3) + (perpendV[0] * -.12), point[1] + (unitV[1] * -0.3) + (perpendV[1] * +0.12)]); // y-positive moves UP
                this.localLineTo(point); // move to end of line segment
                this.ctx.fillStyle = 'blue';
                this.ctx.fill();
            }
            if (markerSymbol == 'dot' || markerSymbol == 'arrowdot') {
                // first the dot at the beginning of the line
                this.ctx.beginPath();
                let center = this.localXY([lastX, lastY]);
                this.ctx.arc(center[0], center[1], 5, 0, 2 * Math.PI);
                this.ctx.fillStyle = 'yellow';
                this.ctx.fill();
                this.ctx.lineWidth = 1;
                this.ctx.strokeStyle = '#003300';
                this.ctx.stroke();
                // then the dot at the end of the line
                this.ctx.beginPath();
                center = this.localXY(point);
                this.ctx.arc(center[0], center[1], 5, 0, 2 * Math.PI);
                this.ctx.fillStyle = 'yellow';
                this.ctx.fill();
                this.ctx.lineWidth = 1;
                this.ctx.strokeStyle = '#003300';
                this.ctx.stroke();
            }
            lastX = point[0]; // set up for next line 
            lastY = point[1];
        });
        // finally
        this.handle.dynamicTexture.update();
    }
    drawCircle(point, radius) {
        this.ctx.beginPath();
        let center = this.localXY(point);
        // draw an ellipse, because a circle won't work if sX !== sY
        this.ctx.ellipse(center[0], center[1], radius * this.sX, radius * this.sY, 0, 0, 2 * Math.PI);
        this.ctx.fillStyle = 'yellow';
        this.ctx.fill();
        this.ctx.lineWidth = 1;
        this.ctx.strokeStyle = '#003300';
        this.ctx.stroke();
        // finally
        this.handle.dynamicTexture.update();
    }
    drawRect(point1, point2) {
        console.log('drawRectangle', point1, point2);
        this.ctx.beginPath();
        this.ctx.lineWidth = 3;
        this.localMoveTo(point1); // start at lower left (might be upper right if numbers are negative)
        this.localLineTo([point1[0], point2[1]]); // up
        this.localLineTo([point2[0], point2[1]]); // right
        this.localLineTo([point2[0], point1[1]]); // down
        this.localLineTo(point1); // close
        this.ctx.stroke();
        // finally
        this.handle.dynamicTexture.update();
    }
    localXY(point) {
        // console.log('localXY', point)
        return [point[0] * this.sX + this.zeroX, point[1] * this.sY * -1 + this.zeroY]; // y-positive moves UP
    }
    localMoveTo(point) {
        // console.log('localMoveTo', point)
        let localPoint = this.localXY(point);
        this.ctx.moveTo(localPoint[0], localPoint[1]);
    }
    localLineTo(point) {
        // console.log('localLineTo', point)
        let localPoint = this.localXY(point);
        this.ctx.lineTo(localPoint[0], localPoint[1]);
    }
}
exports.GRAPH = GRAPH;
class CameraThing extends ThingCommon {
    constructor(cmd, handle, scene) {
        super(cmd, handle, scene);
        console.assert(handle.babylonMesh !== undefined);
        let c = new babylonjs_1.ArcRotateCamera(handle.uniqueName, 0, 0, 15, new babylonjs_1.Vector3(0, 0, 0), this.scene);
        handle.babylonMesh = c;
        c.setPosition(new babylonjs_1.Vector3(0, 0, 20));
        // c.setPosition(new Vector3(5, 20, -20))
        // c.setPosition(new Vector3(0, 0, 15))
        c.useAutoRotationBehavior = false;
        handle.type = 'camera';
        // // This attaches the camera to the canvas
        c.attachControl('', false);
        if (cmd.cameraType == 'follow') {
            const targetMeshHandle = babyengine_1.BabyEngine.shapeMap.get(cmd.otherUuid);
            c.lockedTarget = targetMeshHandle.babylonMesh;
        }
        scene.setActiveCameraByName(handle.uniqueName);
        console.assert(handle.babylonMesh !== undefined);
    }
}
exports.CameraThing = CameraThing;
class Floor extends ThingCommon {
    // a floor is actually a thin box
    constructor(cmd, handle, scene) {
        super(cmd, handle, scene);
        this.handle.babylonMesh = babylonjs_1.MeshBuilder.CreateBox(handle.uniqueName, this.params, this.scene);
        this.standardMeshSetup();
        this.handle.babylonMesh.scaling = new babylonjs_1.Vector3(cmd.x, cmd.y, 0.1);
        this.handle.babylonMesh.rotate(new babylonjs_1.Vector3(1, 0, 0), Math.PI / 2);
        // this.handle.babylonMesh.rotate(new Vector3(0, 1, 0), Math.PI )
        this.handle.babylonMesh.position.z = -0.5; // move to match cube footprint
        // floor has mass 0, always passive (set up Impostor, no callback)
        this.handle.mass = 0;
        this.handle.restitution = .3;
        this.handle.collider = 'solid';
        this.impostorString = 'BoxImpostor';
        this.createPhysicsImpostor();
        this.handle.babylonMesh.material.dispose(); // get rid of the standard material
        let gridMaterial = new babylonjs_materials_1.GridMaterial('grid', this.scene);
        gridMaterial.gridRatio = 1 / cmd.x;
        gridMaterial.majorUnitFrequency = cmd.major;
        if (handle.babylonMesh instanceof babylonjs_1.Mesh) { // typeguard
            handle.babylonMesh.material = gridMaterial;
        }
        else
            console.error('should be Mesh');
        // this.setupCollider()
    }
}
exports.Floor = Floor;
//# sourceMappingURL=things.js.map