"use strict";
/* eslint-disable @typescript-eslint/no-unused-vars */
/* eslint-disable @typescript-eslint/ban-types */
/* eslint-disable @typescript-eslint/no-inferrable-types */
Object.defineProperty(exports, "__esModule", { value: true });
exports.Baby = exports.numbers = exports.range = void 0;
//
// the game source is only available for player0.
// it manipulates 'Baby' and 'BabyMesh' which send
// serialized messages but do not touch the babylon engine
// messages go to BabyEngine, a single object that
// manages an array of meshes, and handles all interactions
// with BabylonShapes, based on those messages.
// BabylonShapes is the only program that 'knows' about BabylonJS.
// presumably I could rewrite it and support a different engine.
// Baby also does setup and provides a bit of game-writer UI
// import { SignBoard } from './prefabs/signboard'
// import { Hinges } from './prefabs/hinges'
// import { AxisLines } from './prefabs/axislines'
const unittest_1 = require("./unittest");
// import { HTMLColor } from './components/htmlcolor'
// import { csgDemo } from './components/csg'
const babything_1 = require("./babything");
// import {GRAPH} from  
const babyGUI_1 = require("./babyGUI");
const babything_2 = require("./babything");
const babyengine_1 = require("../middle/babyengine");
const babylonjs_1 = require("babylonjs");
/** Creates an array of n elements, 0,1,2... */
let range = (start, stop, step = 1) => {
    if (stop === undefined) { // range(5) means range(0,5)
        stop = start || 0;
        start = 0;
    }
    // set up the calculations
    let length = Math.max(Math.ceil((stop - start) / step), 0);
    let i = 0;
    let temp = new Array(length);
    // ok, smear the numbers into the array
    while (i < length) {
        temp[i++] = start;
        start += step; // no problem if step is negative
    }
    return temp;
};
exports.range = range;
/** generates iterable stream of numbers, use this to replace for(let i=0; i< n; i++) */
function* numbers(first, max, step = 1) {
    if (max == undefined) {
        max = first;
        first = 0;
    }
    while (first < max) {
        yield first;
        first += step;
    }
}
exports.numbers = numbers;
/** For many games, `Baby()` is the only class you need to import.
 *
 * ```ts
 * import { Baby } from 'Baby'
 * app = new Baby()    // create an instance of Baby<br><br>
 * ```
 * @remarks
 *
 *
 * Then create the objects and events you need with methods like:
 *
 *  ```ts
 * floor = app.floor(20,20)               // lay down a floor
 * cube = app.shape('cube').color('red')  // add a red cube
 *
 * // hook the keyboard, and move the cube on each keypress
 * app.keydown((data)=>{ cube.move('right',1) })
 * ```
 *
 * <br><br>That's a pretty simple game, but it runs just fine.  We
 * explain it in the 'Getting  Started' tutorial.
 */
class Baby {
    /** Start here.  `Baby()` is the only class you need to import.
     *
     * ```ts
     * import { Baby } from 'wherever'
     * app = new Baby()    // create an instance of the Baby engine.
     * ```
     *
     * The optional parameter sends code to the game.  It's only intended for writing tutorials.
     */
    constructor(src = undefined) {
        /** @ignore */
        this.runUnittests = false;
        /** @ignore */
        this.onSignal = (msg, value) => { }; // allows an outside process to send a signal to a game
        /** @ignore */
        this.uuidValue = 0;
        this.babyEngine = new babyengine_1.BabyEngine(); // BabyEngine, before anything else
        console.log('starting baby...');
        // this.camera() //'arcRotate', this.origin, 20, 10, 20)   // default camera
        if (src !== undefined) {
            console.log('arrived in Baby() with payload', src);
            let app = this;
            eval(src); // yes, eval is crazy unsafe
            return;
        }
        // can find the home position, eg:  pointAt(app.origin)
        const origin = this.shape('cube').opacity(0, 0).collide('ghost'); // create in zero time
        // csgDemo(this.scene, this.canvas)
        /*

                  // // create a label card
                  // let sign = new SignBoard(5, 3, new Vector3(-5, 5, 1), this.scene);
                  // sign.drawText('Hinge', 2, 40, "Bold 40px monospace", "green");
                  // sign.drawText('hello world', 4, 70, "20px Arial", "blue");

                  // // add some axis lines (great for development)
                  // let axis = new AxisLines()
                  // axis.globalAxis(5, Vector3.Zero(), this.scene);
                  // axis.globalAxis(5, new Vector3(3, 3, 3), this.scene);

                  // // fire up a simple hinge
                  // let hinges = new Hinges();
                  // let myHinge = hinges.hinge(Vector3.Zero(), this.scene);
          */
        /// ///////////////////////////////////////////////////////
        // calling unittest this way runs the test every time we run (good when working on a function)
        // this runs every unittest when we click the 'unittest' button
        // this.graphTest()
        // this.numberAt('5', -3, -3, 'yellow') 
        // #!if unittest!==true
        if (!this.runUnittests) {
            // nothing here, this is what happens if no preprocessor
        }
        else {
            // #!endif
            // this.test()
            // let test = new Unittests(this)
            // test.babyShapeTester(this, 'turn')
            // test.spiral()
            // test.testPhysics(this)
            // the unittest module will be removed by the tree-shaker in production. 
            this.GUI('button')
                .position('BottomRight')
                .text('Unittests')
                .onClick((playerN) => new unittest_1.Unittests(this).unittestAll()); // leave them right in the code for now
            // #!if unittest!==true
        }
        // #!endif
        /// ///////////////////////////////////////////////////////
        return (this);
    }
    async numberAt(text, x, y, color) {
        let numberCube = this.cube().color(color, 0);
        let ctx = numberCube.getContext();
        numberCube.size([2, 2, 2]);
        numberCube.turn('right', 2);
        numberCube.move('right', x);
        numberCube.move('up', y).done();
        let params = { 'text': text };
        numberCube.setTexture(params, () => {
            ctx.fillStyle = 'yellow';
            ctx.beginPath();
            ctx.rect(0, 0, 1024, 1024); // preset to 1024
            ctx.fill();
            ctx.beginPath();
            ctx.moveTo(200, 200);
            ctx.fillStyle = 'blue';
            ctx.font = "512px Arial";
            ctx.fillText(params['text'], 300, 600);
        });
        // this returns a promise, lets the caller 'await'
        return numberCube.turn('left', 2.25, .5).done();
    }
    async graphTest() {
        let graph = this.Graph(20, 16).drawGrid().line('arrowdot', [2, 2], [3, 3], [1, 3]).circle([-2, -2], 2).rectangle([-3, -4], [-1, -1]);
        graph.textAt('hello', -5, -5);
        // let floor = this.floor(20,20)
        let camera = this.camera().attachControls(); //.moveToZero().move('up',15)
        let x = this.cube().color('red').addAxes();
        // let c = this.camera().setOrthographic(10)
        x.penDown().move('up', 4).move('forward', 1);
        // let a = this.graph()
        // let b = this.cube().color('yellow').addAxes()
        // let vt = this.vt52()
        // vt.print('HELLO WORLD')
        // vt.print('hello world', "red")
        // for (let i = 0; i < 60; i++) {
        //     vt.print(i.toString())
        //     vt.print('\n')
        // }
        // for (let i = 0; i < 8; i++) {
        //     for (let j = 0; j < 10; j++) {
        //         vt.print(j.toString())
        //     }
        // }
        // for (let i = 0; i < 10; i++) {
        //     vt.print(i.toString())
        //     vt.print('\n')
        // }
        // await vt.print('done')
        // console.log('done')
        // this.activeCameraControls(true)
        // let myInput = (await vt.input())
        // console.log('return from await input', myInput)
        // let name = await vt.input("what's your name?")
        // vt.print('hello ' + name)
    }
    // async loader() {
    //     // The first parameter can be used to specify which mesh to import. Here we import all meshes
    //     let a = new GLTFFileLoader()
    //     await a.importMeshAsync('', this.scene, 'TinkerCraft.glb', "assets/",)
    //     //    Parameters
    //     //    meshesNames: any
    //     //    An array of mesh names, a single mesh name, or empty string for all meshes that filter what meshes are imported
    //     //    scene: Scene
    //     //    The scene to import into
    //     //    data: any
    //     //    The data to import
    //     //    rootUrl: string
    //     //    The root url for scene and resources
    // }
    version() {
        return 'unknown';
    }
    /** @ignore */
    signal(msg, value = 0) {
        this.onSignal(msg, value); // which is usually inside an eval()
    }
    /** freezes the scene while keeping the engine running; expecially useful with the inspector.
    * @remarks
    * ```
    * Add a button to freeze your sceen:
    *    let frozen = false
    *    app.GUI('button')
    *        .position('BottomLeft')
    *        .text('Freeze')
    *        .onClick(() => {
    *            app.freeze(!frozen)
    *            frozen = !frozen;
    *        })
    * ```
    */
    freeze(onOff = true) {
        babyengine_1.BabyEngine.frozen = onOff;
        return this;
    }
    /** trap keydown, look at the data object for ctrl-, alt-, etc */
    keydown(keyFunction) {
        babyengine_1.BabyEngine.addObserver('keydown', (data) => {
            keyFunction(data);
        });
        return this;
    }
    /** trap keypress, look at the data object for ctrl-, alt-, etc */
    keypress(keyFunction) {
        babyengine_1.BabyEngine.addObserver('keypress', (data) => {
            keyFunction(data);
        });
        return this;
    }
    /** trap keyup, look at the data object for ctrl-, alt-, etc */
    keyup(keyFunction) {
        babyengine_1.BabyEngine.addObserver('keyup', (data) => {
            keyFunction(data);
        });
        return this;
    }
    multiPlayer() {
        return this;
    }
    /////////////////////////////////////////////////
    //  these are the creation methods for 'basic'
    // shapes, floors, loaders, csgs, etc.  
    /////////////////////////////////////////////////
    /** Create an 'Shape', which is a single graphic element.  `Baby.Shape('Cube') is the same as calling `Baby.Cube()`,
     * but not all shapes have direct calls and you can add new shapes.
     * The standard shapes are cube, sphere, cylinder, cone, torus, capsule, point, and floor. */
    shape(model) {
        const newE = new babything_1.BabyShape(model, this.babyEngine); // creates the game-writer's object
        const gameCmd = {
            action: 'createBabylonMesh',
            uuid: newE.uuid,
            model: model,
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd); // eventually creates the babylonjs object
        return (newE);
    }
    /** Create a Cube (which you can resize into a cuboid) */
    cube() { return (this.shape('cube')); }
    /** Create a Sphere (which you can resize into an spheroid */
    sphere() { return (this.shape('sphere')); }
    /** Create a Cylinder */
    cylinder() { return (this.shape('cylinder')); }
    /** Create a Cone (strictly speaking, an cone is a cylinder) */
    cone() { return (this.shape('cone')); }
    /** Create a Capsule */
    capsule() { return (this.shape('capsule')); }
    /** Create a Torus */
    torus() { return (this.shape('torus')); }
    /** Create a Point (think of a sphere that is too small to see) */
    point() { return (this.shape('point')); }
    /** Create a terminal plane modelled on the classic VT52 terminal.
     * Underneath, it's really a cube, with a pair of prints functions.
     *
     * We DETACH the camera controls from the canvas when you create a VT52
     * because that's almost always right.  If you want to use the controls
     * then simply turn them back on AFTER you create the VT52 instance
     * eg:   app.activeCameraControls(true)
     */
    VT52() {
        const newE = new babything_1.Vt52Thing('vt52', this.babyEngine); // creates the game-writer's object
        const gameCmd = {
            action: 'createBabylonMesh',
            uuid: newE.uuid,
            model: 'vt52',
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd); // eventually creates the babylonjs object
        return (newE);
    }
    /** Create a 2D Graph plane. Underneath, it's really a cube of size x * y).
     * To create the grid, use  `let g = app.Graph(20,16).drawGrid()
     *
     * We DETACH the camera controls from the canvas
     * because that's almost always right.  If you want to use the controls
     * then simply turn them back on AFTER you create the Graph instance
     * eg:   app.activeCameraControls(true)
     */
    Graph(x, y) {
        const newE = new babything_1.BabyGraph('graph', this.babyEngine); // creates the game-writer's object
        const gameCmd = {
            action: 'createBabylonMesh',
            uuid: newE.uuid,
            model: 'graph',
            x: x,
            y: y
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd); // eventually creates the babylonjs object
        return (newE);
    }
    importShape(directory, filename) {
        const model = 'import';
        const newE = new babything_1.BabyShape(model, this.babyEngine); // creates the game-writer's object
        const gameCmd = {
            action: 'importShape',
            uuid: newE.uuid,
            directory: directory,
            filename: filename,
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd); // eventually creates the babylonjs object
        return (newE);
    }
    /** Add a floor object with  a default grid. Use .collide('solid') if you want to bounce off it. */
    floor(xSize, ySize, major = 1) {
        const newE = new babything_1.BabyShape('floor', this.babyEngine); // creates the game-writer's object
        const gameCmd = {
            action: 'createBabylonMesh',
            uuid: newE.uuid,
            model: 'floor',
            x: xSize,
            y: ySize,
            major: major
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd); // eventually creates the babylonjs object
        return (newE);
    }
    //    // default camera
    //    const camera = new ArcRotateCamera('Camera', 0, 0, 15, new Vector3(-0, -5, 0), this.scene)
    //    camera.setPosition(new Vector3(5, 20, -20))
    //    camera.minZ = 5.0
    //    camera.attachControl(this.canvas, true)
    // for follow, see https://www.babylonjs-playground.com/#0JK6VN#31
    /** A 'camera' defines the player, a multi-player game may have multiple cameras.
     * This is the default 3D camera, look at `OrthoCamera` for 2D
    */
    camera() {
        // TODO: Lots more work required here
        const newE = new babything_2.BabyCamera('camera', this.babyEngine); // creates the game-writer's object
        // console.log ("camera handle ", newE.uuid)
        const gameCmd = {
            action: 'setCamera',
            uuid: newE.uuid,
            model: 'arcRotate',
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd); // eventually creates the babylonjs object
        return (newE);
    }
    // orthoCamera(){
    //     const newE = new BabyCamera('camera', this.babyEngine) // creates the game-writer's object
    //     console.log ("OrthoCamera handle ", newE.uuid)
    //     const gameCmd: gameCmd = {
    //         action: 'setCamera',
    //         uuid: newE.uuid,
    //         model: 'orthocamera',
    //     }
    //     BabyEngine.broadcastToPlayers(gameCmd) // eventually creates the babylonjs object
    //     return (newE)
    // }
    /** Turn on the BabylonJS Inspector.  Only for player0.
     * only valid parameter at this point is 'record', which captures the first 7 seconds of your game in video
    */
    inspector(stringParm = '') {
        // TODO: is this player0 ??
        // this.scene.debugLayer.show()
        const gameCmd = {
            action: 'inspector',
            uuid: -1,
            stringParm: stringParm
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd); // eventually creates the babylonjs object
        return (this);
    }
    /// ///////////
    /// some utilities
    /// //////////
    // TODO
    /** random integer between min (includes) and max (excludes).  so randomIntBetween(2,5) might return 2,3, or 4, but not 5. */
    randomIntBetween(min, max) {
        min = Math.ceil(min);
        max = Math.floor(max);
        return Math.floor(Math.random() * (max - min) + min); // The maximum is exclusive and the minimum is inclusive
    }
    ////////////////////////////// buttons  ////////////////
    /** Create a 'Button'.  */
    GUI(model) {
        // role: 'Primary' | 'Secondary' | 'Success' | 'Danger' | 'Warning' | 'Info' | 'Light' | 'Dark' = 'Primary',
        // position: 'Top' | 'TopLeft' | 'TopRight' | 'Left' | 'Right' | 'Bottom' | 'BotRight' | 'BotLeft' | 'Center' = 'Center'): BabyGUI {
        const newE = new babyGUI_1.BabyGUI(model, this, this.babyEngine); // creates the game-writer's object
        const gameCmd = {
            action: 'createGUI',
            uuid: newE.uuid,
            model: model,
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd); // eventually creates the babylonjs object
        return (newE);
    }
    /** dispose of the scene, releasing all meshes, animations, etc */
    dispose() {
        const gameCmd = {
            uuid: 0,
            action: 'gameDispose',
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd); // eventually creates the babylonjs object
    }
    /** attach or detach the current camera controls from the canvas */
    activeCameraControls(on) {
        // getActiveCamera():{
        let handle = babyengine_1.BabyEngine.babylonShapes.activeCameraHandle;
        if (handle.babylonMesh instanceof babylonjs_1.Camera) {
            if (on)
                handle.babylonMesh.attachControl();
            else
                handle.babylonMesh.detachControl();
        }
    }
}
exports.Baby = Baby;
//# sourceMappingURL=baby.js.map