"use strict";
// buttons are GUI items.   
Object.defineProperty(exports, "__esModule", { value: true });
exports.BabyGUI = void 0;
const babything_1 = require("./babything");
const babyengine_1 = require("../middle/babyengine");
// you position them FIRST in (N, NE, E SE... corners)  // default is CENTER
// then you can move them up/down or left/right
// we follow BOOTSTRAP's naming and color scheme...
// https://getbootstrap.com/docs/4.0/components/buttons/   
class BabyGUI extends babything_1.BabyThing {
    constructor(model, baby, babyEngine) {
        super(model, babyEngine);
    }
    /** role sets the color, eg Primary is blue, Danger is red.  This follows Bootstrap's model. */
    role(role) {
        switch (role) {
            case 'Primary':
                this.color = '#fff';
                this.background = '#007bff';
                this.border = '#007bff';
                break;
            case 'Secondary':
                this.color = '#fff';
                this.background = '#6c757d';
                this.border = '#6c757d';
                break;
            case 'Success':
                this.color = '#fff';
                this.background = '#28a745';
                this.border = '#28a745';
                break;
            case 'Danger':
                this.color = '#fff';
                this.background = '#dc3545';
                this.border = '#dc3545';
                break;
            case 'Warning':
                this.color = '#212529';
                this.background = '#ffc107';
                this.border = '#ffc107';
                break;
            case 'Info':
                this.color = '#fff';
                this.background = '#17a2b8';
                this.border = '#17a2b8';
                break;
            case 'Light':
                this.color = '#212529';
                this.background = '#f8f9fa';
                this.border = '#17a2b8';
                break;
            case 'Dark':
                this.color = '#fff';
                this.background = '#343a40';
                this.border = '#343a40';
                break;
            default:
                console.error(`did not expect '${role}' as a button role`);
        }
        const gameCmd = {
            action: 'colorGUI',
            uuid: this.uuid,
            color: this.color,
            background: this.background,
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd);
        return this;
    }
    /** position is one of the eight side/corner positions or 'Center'. eg: 'TopLeft' */
    position(position) {
        switch (position) {
            case 'Top':
                this.leftRight = 0;
                this.topBottom = 1;
                break;
            case 'TopRight':
                this.leftRight = 1;
                this.topBottom = 1;
                break;
            case 'TopLeft':
                this.leftRight = -1;
                this.topBottom = 1;
                break;
            case 'Right':
                this.leftRight = 1;
                this.topBottom = 0;
                break;
            case 'Left':
                this.leftRight = -1;
                this.topBottom = 0;
                break;
            case 'Bottom':
                this.leftRight = 0;
                this.topBottom = -1;
                break;
            case 'BottomRight':
                this.leftRight = 1;
                this.topBottom = -1;
                break;
            case 'BottomLeft':
                this.leftRight = -1;
                this.topBottom = -1;
                break;
            case 'Center':
                this.leftRight = 0;
                this.topBottom = 0;
                break;
            default:
                console.error(`did not expect '${position}' as a button position`);
        }
        const gameCmd = {
            action: 'positionGUI',
            uuid: this.uuid,
            leftRight: this.leftRight,
            topBottom: this.topBottom,
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd);
        return this;
    }
    /** the message to display */
    text(text, resize = true) {
        const gameCmd = {
            action: 'textGUI',
            uuid: this.uuid,
            text: text,
            resize: true,
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd);
        return this;
    }
    onClick(func) {
        this.onPressDown = func;
        return this;
    }
}
exports.BabyGUI = BabyGUI;
//# sourceMappingURL=babyGUI.js.map