"use strict";
// this provides the methods for 'BabyShape' objects.  It doesn't actually do
// anything, simply dispaches methods to BabyEngine.
Object.defineProperty(exports, "__esModule", { value: true });
exports.BabyCamera = exports.BabyGraph = exports.Vt52Thing = exports.BabyShape = exports.BabyThing = void 0;
const tween_1 = require("../middle/tween");
const babyengine_1 = require("../middle/babyengine");
const babylonjs_1 = require("babylonjs");
const utilities_1 = require("./utilities");
const consoleRed = 'background-color:red;color:white;';
const consoleYellow = 'background-color:yellow;color:black;';
const consoleBlue = 'background-color:blue;color:white;';
/** superclass for BabyShape, BabyCamera and BabyButton */
class BabyThing {
    constructor(model, babyEngine) {
        this.model = model;
        // this.baby = baby
        this.babyEngine = babyEngine;
        let temp = new utilities_1.Uuid(); // increments 
        this.uuid = utilities_1.Uuid.count; // our unique id - VERY IMPORTANT
        this.collideCallback = (otherBabyMesh) => { return this; }; // empty function unless overridden
        this.clickCallback = () => { return this; }; // empty function unless overridden
        this.onPressDown = (playerN) => {
            console.log('press down for player ', playerN);
            return this;
        }; // empty function unless overridden
        // the handle is still empty, but register it anyway
        const handle = new babyengine_1.MeshHandle(model, this.uuid);
        babyengine_1.BabyEngine.shapeMap.set(this.uuid, handle); // register this shape object into the meshHandle
        handle.babyShape = this; // add a pointer back to ourself
    }
    // some functions work on all kinds of things. so they are here in the 'parent'
    //TODO: figure why this doesn't work
    animatedGif(resultion = 200) {
        var _a;
        const gameCmd = {
            action: 'animatedGIF',
            uuid: this.uuid
        };
        this.lastTween = (_a = babyengine_1.BabyEngine.broadcastToPlayers(gameCmd)) !== null && _a !== void 0 ? _a : this.lastTween;
        return this;
    }
    /** Returns a promise that resolves when the LAST issued tween is completed.  Use it as  `await thing.move(...).done()` */
    async done() {
        // console.log('in done, tween is ',this.lastTween)
        return new Promise(resolve => {
            // we are looking for a specific tween (because an object may have many tweens running)
            // but sometimes we can't figure out which one it is.   Just alert the programmer.
            if (typeof (this.lastTween) == 'undefined')
                console.error(`We don't know which tween (if any) you are waiting for. Did you mean to use shape.isIdle()?`);
            // we store it away, run it when the tween ends
            this.lastTween.promiseResolve = resolve;
        });
    }
    /** Returns a promise that resolves when no tweens remain active.  Use it as  `await thing.isIdle()` */
    async isIdle() {
        return new Promise(resolve => {
            // we are looking for a specific tween (because an object may have many tweens running)
            // but sometimes we can't figure out which one it is.   Just alert the programmer.
            let tween = new tween_1.MiscTween('misc');
            let handle = babyengine_1.BabyEngine.shapeMap.get(this.uuid);
            tween.promise = new Promise(() => {
                tween.start(() => { return (handle.tweenArray.length <= 1); }); // 1 because THIS tween is still there
            });
            handle.tweenArray.push(tween);
            // we store it away with current context, run it when the tween ends
            tween.promiseResolve = resolve;
        });
    }
}
exports.BabyThing = BabyThing;
// general shape of Babylon meshbuilder is
// var shape = MeshBuilder.CreateShape(name, options, scene)
class BabyShape extends BabyThing {
    constructor(model, babyEngine) {
        super(model, babyEngine);
    }
    /** a callback when an ACTIVE shape collides with an ACTIVE or PASSIVE shape
     * note: if two ACTIVE shapes collide, they BOTH get a callback notification
     * eg:  myBall.onCollide((other)=>{      // other is a BabyMesh object
     *               if(other == hisBall){  doSomething()  }
     *         })
     */
    onCollide(callback) {
        // store the callback function.
        // then simply subscribe to an observer event called "collide.nnn" where
        // nnn is our uuid.  when the event happens, trigger the callback
        // yes, it would be simplier to tie the callback to the physics engine,
        // but we are trying to isolate the baby side from the babylon side
        this.collideCallback = callback;
        babyengine_1.BabyEngine.addObserver(`collide.${this.uuid}`, (data) => {
            // console.log(`onCollide() heard collide.${this.uuid}`, data)
            this.collideCallback(data);
        });
        return this;
    }
    /** a callback when the mouse clicks this shape.
     * eg:  myBall.onClick(()=>console.log("I was clicked"))
     */
    onClick(callback) {
        // store the callback function.
        // then simply subscribe to an observer event called "collide.nnn" where
        // nnn is our uuid.  when the event happens, trigger the callback
        // yes, it would be simplier to tie the callback to the physics engine,
        // but we are trying to isolate the baby side from the babylon side
        this.clickCallback = callback;
        console.log('adding observer', `click.${this.uuid}`);
        babyengine_1.BabyEngine.addObserver(`click.${this.uuid}`, () => {
            this.clickCallback(); // usually an empty function
        });
        return this;
    }
    /*
        setPosition(v: number[]): BabyMesh {
            console.assert(v.length == 3, `expected a vector of 3 numbers in addToPosition()`)

            let gameCmd: gameCmd = {
                action: 'setPosition',
                uuid: this.uuid,
                x: v[0],
                y: v[1],
                z: v[2]
            }
            BabyEngine.broadcastToPlayers(gameCmd)
            return (this)
        }

        addToPosition(v: number[]): BabyMesh {
            console.assert(v.length == 3, `expected a vector of 3 numbers in addToPosition()`)

            let gameCmd: gameCmd = {
                action: 'addToPosition',
                uuid: this.uuid,
                x: v[0],
                y: v[1],
                z: v[2]
            }
            BabyEngine.broadcastToPlayers(gameCmd)
            return (this)
        }


        // the game writer does not have access to this
        private getBabylonPosition(): [number, number, number] {
            let bMesh = this.babylonMeshs.entityMap.get(this.uuid).babylonMesh
            console.assert(typeof bMesh !== undefined)  // timing?
            return [bMesh.position.x, bMesh.position.y, bMesh.position.z]
        }

        // setScale(v: number[]): BabyMesh {
        //     console.assert(v.length == 3, `expected a vector of 3 numbers in setScale()`)

        //     this.scaling = new Vector3(v[0], v[1], v[2])

        //     this.babylonEntity.scaling = this.scaling  // no need to recreate
        //     return (this)
        // }

        // // TODO: only player0 can use this, need another mechanism
        // getScale(): number[] {  // don't use Vector3 in userland
        //     return [this.scaling.x, this.scaling.y, this.scaling.z]
        // }

        // setRotation(p: number[]): BabyMesh {
        //     console.assert(p.length == 3, `setRotation expects an array of 3 values [x,y,z]`)

        //     this.rotation = new Vector3(p[0], p[1], p[2])

        //     this.babylonEntity.rotation = this.rotation  // no need to recreate
        //     return (this)
        // }

        // getRotation(): number[] {  // don't use Vector3 in userland
        //     return [this.rotation.x, this.rotation.y, this.rotation.z]
        // }

        // setGrid(gridRatio: number = 0.1, majorUnitFrequency: number = 2, lineColor: string = "White"): BabyMesh {

        //     let gameCmd: gameCmd = {
        //         action: 'setColor',
        //         uuid: this.uuid,
        //         gridRatio: gridRatio,
        //         major: majorUnitFrequency,
        //         colorD: lineColor
        //     }
        //     BabyEngine.broadcastToPlayers(gameCmd)
        //     return (this)
        // }

    //  **
    //  * Sets diffuse color (the essential color under white light)
    //  * ambient color (the color when in shadow)
    //  * emmissive color (the self-illumination color)
    //  * specular color (the shiny-reflections color)
    //  * use the empty string ('') to skip a color
    //  * @param {diffuse} string of the 140 HTML cololr names
    //  * @param {ambient} (optional) any of the 140 HTML color names
    //  * @param {emmisive} (optional) any of the 140 HTML color names
    //  * @param {specular} (optional) any of the 140 HTML color names
    //  *
        setColor(diffuse: string, ambient: string = '', emmisive: string = '', specular: string = ''): BabyMesh {

            let gameCmd: gameCmd = {
                action: 'setColor',
                uuid: this.uuid,
                colorD: diffuse,
                colorA: ambient,
                colorE: emmisive,
                colorS: specular,
            }
            BabyEngine.broadcastToPlayers(gameCmd)
            return (this)
        }

    */
    /* add axes to a shape for debugging purposes. Blue is forward, red is right, green is up. */
    addAxes() {
        const gameCmd = {
            action: 'addAxes',
            uuid: this.uuid
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd);
        return (this);
    }
    /// ////////////////////////////////
    ///  Alice Functions
    /// ////////////////////////////////
    /** dispose of this shape or camera. */
    dispose() {
        const gameCmd = {
            action: 'dispose',
            uuid: this.uuid
        };
        /*this.lastTween = */ babyengine_1.BabyEngine.broadcastToPlayers(gameCmd);
        return (this);
    }
    /** turn collisions on or off.  'active' will turn on gravity and mass. */
    collide(collider = 'ghost') {
        //TODO: don't enable physics engine until there is at least one solid or active thing
        const gameCmd = {
            action: 'collide',
            uuid: this.uuid,
            collider: collider
        };
        // don't need a tween, so don't overwrite 'lastTween'
        console.assert(this.lastTween !== null, "Don't 'await' setting a collide, it happens instantly.");
        /*this.lastTween = */ babyengine_1.BabyEngine.broadcastToPlayers(gameCmd);
        return (this);
    }
    /**  Move an entity  */
    move(direction, distance, duration = 1, animate = '1-Begin And End Normally') {
        var _a;
        const gameCmd = {
            action: 'move',
            uuid: this.uuid,
            duration: duration,
            animate: animate,
            direction: direction,
            distance: distance
        };
        // update lastTween if we get a new tween.  sometimes we only get a null
        this.lastTween = (_a = babyengine_1.BabyEngine.broadcastToPlayers(gameCmd)) !== null && _a !== void 0 ? _a : this.lastTween;
        return (this);
    }
    /**  MoveAndTurn - moves in an arc (the turn affects the move)  */
    moveAndTurn(moveDirection, distance, turnTowards, angle, duration = 1, animate = '1-Begin And End Normally') {
        var _a;
        const gameCmd = {
            action: 'moveAndTurn',
            uuid: this.uuid,
            duration: duration,
            animate: animate,
            direction: moveDirection,
            distance: distance,
            towards: turnTowards,
            angle: angle
        };
        // update lastTween if we get a new tween.  sometimes we only get a null
        this.lastTween = (_a = babyengine_1.BabyEngine.broadcastToPlayers(gameCmd)) !== null && _a !== void 0 ? _a : this.lastTween;
        return (this);
    }
    /**  MoveToward- moves towards another shape.  Often getDistanceTo(other) is useful.  */
    moveToward(other, distance, duration = 1, animate = '1-Begin And End Normally') {
        var _a;
        const gameCmd = {
            action: 'moveToward',
            uuid: this.uuid,
            duration: duration,
            animate: animate,
            otherUuid: other.uuid,
            distance: distance
        };
        // update lastTween if we get a new tween.  sometimes we only get a null
        this.lastTween = (_a = babyengine_1.BabyEngine.broadcastToPlayers(gameCmd)) !== null && _a !== void 0 ? _a : this.lastTween;
        return (this);
    }
    moveToZero(duration = 1, animate = '1-Begin And End Normally') {
        var _a;
        const gameCmd = {
            action: 'moveToZero',
            uuid: this.uuid,
            animate: animate,
            duration: duration
        };
        // update lastTween if we get a new tween.  sometimes we only get a null
        this.lastTween = (_a = babyengine_1.BabyEngine.broadcastToPlayers(gameCmd)) !== null && _a !== void 0 ? _a : this.lastTween;
        return (this);
    }
    /**  Turn an entity  (1 is a full turn)
     *
     * @remarks
     * You get different results from the two cases below, because in the second case
     * the box waits to finish the first turn before starting the second.  Even if you
     * do the turns in zero-time, you cannot be sure which will be performed first unless
     * you wait. Consider using the 'legacy' function `setRotation()`.
     *
     *
     * ```
     * box.turn('forward',.25)
     * box.turn('left',.25)
     * ```
     * and
     * ```
     * await box.turn('forward',.25).done()
     * box.turn('left',.25)
     * ```
    */
    turn(towards, angle, duration = 1, animate = '1-Begin And End Normally') {
        var _a;
        const gameCmd = {
            action: 'turn',
            uuid: this.uuid,
            towards: towards,
            angle: angle,
            duration: duration,
            animate: animate
        };
        // update lastTween if we get a new tween.  sometimes we only get a null
        this.lastTween = (_a = babyengine_1.BabyEngine.broadcastToPlayers(gameCmd)) !== null && _a !== void 0 ? _a : this.lastTween;
        return (this);
    }
    turnToSame(other, duration = 1, animate = '1-Begin And End Normally') {
        var _a;
        const gameCmd = {
            action: 'turnToSame',
            uuid: this.uuid,
            otherUuid: other.uuid
        };
        // update lastTween if we get a new tween.  sometimes we only get a null
        this.lastTween = (_a = babyengine_1.BabyEngine.broadcastToPlayers(gameCmd)) !== null && _a !== void 0 ? _a : this.lastTween;
        return (this);
    }
    turnToZero(duration = 1, animate = '1-Begin And End Normally') {
        var _a;
        const gameCmd = {
            action: 'turnToZero',
            uuid: this.uuid,
            animate: animate,
            duration: duration
        };
        // update lastTween if we get a new tween.  sometimes we only get a null
        this.lastTween = (_a = babyengine_1.BabyEngine.broadcastToPlayers(gameCmd)) !== null && _a !== void 0 ? _a : this.lastTween;
        return (this);
    }
    /**
     * Size or resize an entity  (same as Babylon 'scale' for simple shapes)
     */
    size(size, duration = 1, animate = '1-Begin And End Normally') {
        var _a;
        let size3;
        if (typeof size === 'number') {
            size3 = [size, size, size];
        }
        else {
            size3 = size;
        }
        const gameCmd = {
            action: 'size',
            uuid: this.uuid,
            x: size3[0],
            y: size3[1],
            z: size3[2],
            duration: duration,
            animate: animate
        };
        // update lastTween if we get a new tween.  sometimes we only get a null
        this.lastTween = (_a = babyengine_1.BabyEngine.broadcastToPlayers(gameCmd)) !== null && _a !== void 0 ? _a : this.lastTween;
        return (this);
    }
    /**
    * Sets diffuse color (the essential color under white light).  For controls
    * on ambient, emmisive, or specular color, use 'setColor()'
    * @param {diffuseColor} string is one of 140 HTML color names eg: 'mediumvioletred'
    */
    color(diffuseColor, duration = 1, animate = '1-Begin And End Normally') {
        var _a;
        const gameCmd = {
            action: 'color',
            uuid: this.uuid,
            colorD: diffuseColor,
            duration: duration,
            animate: animate
        };
        // update lastTween if we get a new tween.  sometimes we only get a null
        this.lastTween = (_a = babyengine_1.BabyEngine.broadcastToPlayers(gameCmd)) !== null && _a !== void 0 ? _a : this.lastTween;
        return (this);
    }
    /** Sets opacity between 0 (transparent) and 1 (solid) */
    opacity(opacity, duration = 1, animate = '1-Begin And End Normally') {
        var _a;
        console.assert(opacity >= 0 && opacity <= 1, "Opacity must be between 0 and 1");
        const gameCmd = {
            action: 'opacity',
            uuid: this.uuid,
            opacity: opacity,
            duration: duration,
            animate: animate
        };
        this.lastTween = (_a = babyengine_1.BabyEngine.broadcastToPlayers(gameCmd)) !== null && _a !== void 0 ? _a : this.lastTween;
        return (this);
    }
    /** Creates a skeleton frame on a shape */
    addSkeleton() {
        var _a;
        const gameCmd = {
            action: 'addSkeleton',
            uuid: this.uuid
        };
        // update lastTween if we get a new tween.  sometimes we only get a null
        this.lastTween = (_a = babyengine_1.BabyEngine.broadcastToPlayers(gameCmd)) !== null && _a !== void 0 ? _a : this.lastTween;
        return (this);
    }
    /** Adds to a shape that has a skeleton */
    attachTo(other) {
        var _a;
        const gameCmd = {
            action: 'attachTo',
            uuid: this.uuid,
            otherUuid: other.uuid
        };
        // update lastTween if we get a new tween.  sometimes we only get a null
        this.lastTween = (_a = babyengine_1.BabyEngine.broadcastToPlayers(gameCmd)) !== null && _a !== void 0 ? _a : this.lastTween;
        return (this);
    }
    /** Kill any in-progress moves (usually because of a collision).  Turns are not affected. 'Await' will fire if anyone is waiting. */
    break() {
        var _a;
        const gameCmd = {
            action: 'break',
            uuid: this.uuid
        };
        // update lastTween if we get a new tween.  sometimes we only get a null
        this.lastTween = (_a = babyengine_1.BabyEngine.broadcastToPlayers(gameCmd)) !== null && _a !== void 0 ? _a : this.lastTween;
        return (this);
    }
    /** Set positon of this shape immediately.  This is the Babylon way of moving shapes.  It is
     * particularly useful for setting an initial position
    */
    setPosition(leftRight, upDown, frontBack) {
        const gameCmd = {
            action: 'setPosition',
            uuid: this.uuid,
            x: leftRight,
            y: upDown,
            z: frontBack
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd);
        return this;
    }
    /** modify positon of this shape immediately.  This is the Babylon way of moving shapes.  */
    addToPosition(leftRight, upDown, frontBack) {
        const gameCmd = {
            action: 'setPosition',
            uuid: this.uuid,
            x: leftRight,
            y: upDown,
            z: frontBack
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd);
        return this;
    }
    /** set direction of this shape immediately.  This is the Babylon way of rotating shapes.
    * The absolute values of the numbers aren't important.  (0,1,0) is the same as (0,100,0).
    * Rotations are performed around the x, y, z axis in order.
    * */
    setRotation(leftRight, upDown, frontBack) {
        const gameCmd = {
            action: 'setRotation',
            uuid: this.uuid,
            x: leftRight,
            y: upDown,
            z: frontBack
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd);
        return this;
    }
    /** modify rotation of this shape immediately.  This is the Babylon way of rotating shapes. A quarter-rotation is .25, as usual */
    addToRotation(leftRight, upDown, frontBack) {
        const gameCmd = {
            action: 'addToRotation',
            uuid: this.uuid,
            x: leftRight,
            y: upDown,
            z: frontBack
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd);
        return this;
    }
    ////// to create a line, we use penDown(), penUp(), lineErase()
    ////// we inherit the color and opacity from the shape using the pen
    /** drop a pen, and start creating a 3d line in space */
    penDown(radius = 0.1) {
        const gameCmd = {
            action: 'penDown',
            uuid: this.uuid,
            radius: radius
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd);
        return this;
    }
    /** raise the pen, stop drawing a 3d line */
    penUp() {
        const gameCmd = {
            action: 'penUp',
            uuid: this.uuid
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd);
        return this;
    }
    /** erase a line created with `penDown` and `penUp` */
    lineErase() {
        const gameCmd = {
            action: 'lineErase',
            uuid: this.uuid
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd);
        return this;
    }
    ////////////////////////////////////////////////
    //////// dynamicTexture 
    ////////////////////////////////////////////////
    /** sets a texture with HTML Canvas commands, for example:
     *   let params = { 'text': '5'}
     *   numberCube.setTexture(params, () => {
     *       ctx.fillStyle = 'yellow';
     *       ctx.beginPath();
     *       ctx.rect(0, 0, 1024, 1024);  // preset to 1024
     *       ctx.fill();
     *
     *       ctx.beginPath();
     *       ctx.moveTo(200, 200)
     *       ctx.fillStyle = 'blue';
     *       ctx.font = "512px Arial";
     *       ctx.fillText(params['text'], 300, 600)
     *   })
    */
    setTexture(params, ctxFunction) {
        // console.log('babything setTexture', ctxFunction.toString())
        const gameCmd = {
            action: 'setTexture',
            uuid: this.uuid,
            params: JSON.stringify(params),
            ctxFunction: ctxFunction.toString()
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd);
        return this;
    }
    // } ) {
    ////////////////////////////////////////////////
    //////// functions (only on player0) ///////////
    ////////////////////////////////////////////////
    // GET and IS functions query the state of the game on the local machine, 
    // and return immediately, they are NOT passed to other machines.
    // where possible, we use Babylon V3 functions
    // these next calls are legacy Babylon, and we call BabylonJS directly
    // only Player0 would be interested in the position of something, so no need to
    // route this call through the multiplayer layer.
    /** position of this thing at this instance */
    getPosition() {
        let handle = babyengine_1.BabyEngine.shapeMap.get(this.uuid);
        if (handle.babylonMesh instanceof babylonjs_1.Mesh) { // typeguard
            return handle.babylonMesh.position;
        }
        else {
            console.error('getPosition()) should only use mesh');
            return babylonjs_1.Vector3.Zero();
        }
    }
    /** direction of this thing at this instance */
    getRotation() {
        let handle = babyengine_1.BabyEngine.shapeMap.get(this.uuid);
        if (handle.babylonMesh instanceof babylonjs_1.Mesh) { // typeguard
            return handle.babylonMesh.rotation;
        }
        else {
            console.error('getScale() should only use mesh');
            return babylonjs_1.Vector3.Zero();
        }
    }
    /** scale of this thing at this instance */
    getSize() {
        let handle = babyengine_1.BabyEngine.shapeMap.get(this.uuid);
        if (handle.babylonMesh instanceof babylonjs_1.Mesh) { // typeguard
            return handle.babylonMesh.scaling;
        }
        else {
            console.error('getScale() should only use mesh');
            return babylonjs_1.Vector3.Zero();
        }
    }
    /** distance to another shape (center-to-center, so watch out if you have resized one of them) */
    getDistanceTo(otherShape) {
        const thisPosition = babyengine_1.BabyEngine.shapeMap.get(this.uuid).babylonMesh.position;
        const otherPosition = babyengine_1.BabyEngine.shapeMap.get(otherShape.uuid).babylonMesh.position;
        return babylonjs_1.Vector3.Distance(thisPosition, otherPosition);
    }
    /** get the HTML context for drawing on this shape
     * eg:  let box = app.cube()
     *      let ctx = box.getContext()
     *      ctx.fillStyle = 'red'
     *      ctx.beginPath()
     *      ctx.rect(0, 0, 100,100)
     *      ctx.fill()
    */
    getContext() {
        let handle = babyengine_1.BabyEngine.shapeMap.get(this.uuid);
        return handle.dynamicTexture.getContext();
    }
}
exports.BabyShape = BabyShape;
class Vt52Thing extends BabyShape {
    constructor(model, babyEngine) {
        super(model, babyEngine);
    }
    /** A VT52 is like an old-fashioned text terminal.  Embed \n in your string for a newline
     *
     * @remarks
     * vt52.print() is an async, so you can use `await vt52.print('done')`.  The
     * promise is resolved when the *LAST* character in the buffer is printed, so
     * you can print multiple statements and only `await` the last one.
     *
     * `Await vt.print()` only waits for the local machine, but all players should take the same time.
     *
     */
    async print(text, color = 'green') {
        const gameCmd = {
            action: 'print',
            uuid: this.uuid,
            text: text,
            color: color
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd); // start the printing
        return new Promise(resolve => {
            let handle = babyengine_1.BabyEngine.shapeMap.get(this.uuid);
            handle.thing['vt52print'] = resolve;
        });
    }
    /** A simple wait-for-input. Make sure you remember the `async` before `vt52.input()
    * async myFunction(){
    *       let userName = await vt52.input("What's your name? ")
    *       vt52.print("Hello " + userName)
    * }
    *
    * You can't use this for multi-player because it doesn't tell you WHO typed.
    */
    async input(prompt = '') {
        if (prompt !== '')
            this.print(prompt); // fires it out to all players
        return new Promise(resolve => {
            let handle = babyengine_1.BabyEngine.shapeMap.get(this.uuid);
            // save this resolve function into the thing.vt object
            handle.thing['vt52input'] = (s) => {
                resolve(s);
            };
        });
    }
}
exports.Vt52Thing = Vt52Thing;
/** A 2D graph class for showing math */
class BabyGraph extends BabyShape {
    constructor(model, babyEngine) {
        super(model, babyEngine);
        this.scaleX = 20;
        this.scaleY = 16;
    }
    /** draw a grid from -x to x, from -y to y (defaulting to the graph size).  */
    drawGrid(x, y) {
        if (x == undefined)
            x = this.scaleX;
        if (y == undefined)
            y = this.scaleY;
        let handle = babyengine_1.BabyEngine.shapeMap.get(this.uuid);
        const gameCmd = {
            action: 'drawGrid',
            uuid: this.uuid,
            x: x,
            y: y
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd);
        return this;
    }
    /** draw a line through a series of points declared as [x,y]
     * eg: graph.line([2,1],[2,2],[3,3])
     */
    line(markerSymbol, start, ...restofPoints) {
        let handle = babyengine_1.BabyEngine.shapeMap.get(this.uuid);
        const gameCmd = {
            action: 'drawLine',
            uuid: this.uuid,
            x: start[0],
            y: start[1],
            marker: markerSymbol,
            rest: JSON.stringify(restofPoints),
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd);
        return this;
    }
    /** draw a circle of radius r at point [x,y]
     * eg: graph.circle([2,1],5)
     */
    circle(point, radius) {
        let handle = babyengine_1.BabyEngine.shapeMap.get(this.uuid);
        const gameCmd = {
            action: 'drawCircle',
            uuid: this.uuid,
            x: point[0],
            y: point[1],
            radius: radius,
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd);
        return this;
    }
    /** draw a rectangle defined by bottom-left and top-right points
    * eg: graph.rectangle([-3,-3],[2,1])
    */
    rectangle(point1, point2) {
        let handle = babyengine_1.BabyEngine.shapeMap.get(this.uuid);
        const gameCmd = {
            action: 'drawRect',
            uuid: this.uuid,
            x: point1[0],
            y: point1[1],
            dx: point2[0],
            dy: point2[1],
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd);
        return this;
    }
    textAt(text, x, y, offsetX = 0, offsetY = 0) {
        let handle = babyengine_1.BabyEngine.shapeMap.get(this.uuid);
        const gameCmd = {
            action: 'textAt',
            uuid: this.uuid,
            text: text,
            x: x,
            y: y,
            dx: offsetX,
            dy: offsetY,
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd);
        return this;
    }
}
exports.BabyGraph = BabyGraph;
class BabyCamera extends BabyThing {
    constructor(model, babyEngine) {
        super(model, babyEngine);
    }
    // move is IDENTICAL to move() for BabyShape
    // except that it returns a BabyCamera
    // I simply don't know how to do this properly in Typescript
    /**  Move an entity  */
    move(direction, distance, duration = 1, animate = '1-Begin And End Normally') {
        var _a;
        const gameCmd = {
            action: 'move',
            uuid: this.uuid,
            duration: duration,
            animate: animate,
            direction: direction,
            distance: distance
        };
        // update lastTween if we get a new tween.  sometimes we only get a null
        this.lastTween = (_a = babyengine_1.BabyEngine.broadcastToPlayers(gameCmd)) !== null && _a !== void 0 ? _a : this.lastTween;
        return (this);
    }
    /**  MoveToward- moves towards another shape.  Often getDistanceTo(other) is useful.  */
    moveToward(other, distance, duration = 1, animate = '1-Begin And End Normally') {
        var _a;
        const gameCmd = {
            action: 'moveToward',
            uuid: this.uuid,
            duration: duration,
            animate: animate,
            otherUuid: other.uuid,
            distance: distance
        };
        // update lastTween if we get a new tween.  sometimes we only get a null
        this.lastTween = (_a = babyengine_1.BabyEngine.broadcastToPlayers(gameCmd)) !== null && _a !== void 0 ? _a : this.lastTween;
        return (this);
    }
    moveToZero(duration = 1, animate = '1-Begin And End Normally') {
        var _a;
        const gameCmd = {
            action: 'moveToZero',
            uuid: this.uuid,
            animate: animate,
            duration: duration
        };
        // update lastTween if we get a new tween.  sometimes we only get a null
        this.lastTween = (_a = babyengine_1.BabyEngine.broadcastToPlayers(gameCmd)) !== null && _a !== void 0 ? _a : this.lastTween;
        return (this);
    }
    /** set camera mode to Orthographic (ie: not Perspective) */
    setOrthographic(orthoSize = 12) {
        const gameCmd = {
            action: 'setOrthographic',
            uuid: this.uuid,
            size: orthoSize
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd);
        return (this);
    }
    /** set camera mode to Orthographic (ie: not Perspective) */
    setActive(player = 0) {
        const gameCmd = {
            action: 'setActive',
            uuid: this.uuid,
            player: player
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd);
        return (this);
    }
    /** gives players control of the camera) */
    attachControls() {
        const gameCmd = {
            action: 'attachControls',
            uuid: this.uuid,
        };
        babyengine_1.BabyEngine.broadcastToPlayers(gameCmd);
        return (this);
    }
}
exports.BabyCamera = BabyCamera;
//# sourceMappingURL=babything.js.map