"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.csgDemo = void 0;
const babylonjs_1 = require("babylonjs");
function csgDemo(scene, canvas) {
    const sourceMat = new babylonjs_1.StandardMaterial('sourceMat', scene);
    sourceMat.wireframe = true;
    sourceMat.backFaceCulling = false;
    const a = babylonjs_1.Mesh.CreateSphere('sphere', 16, 4, scene);
    const b = babylonjs_1.Mesh.CreateBox('cube', 4, scene);
    const c = babylonjs_1.Mesh.CreateBox('cube', 4, scene);
    a.material = sourceMat;
    b.material = sourceMat;
    c.material = sourceMat;
    a.position.y += 5;
    b.position.y += 2.5;
    c.position.y += 3.5;
    c.rotation.y += Math.PI / 8.0;
    const aCSG = babylonjs_1.CSG.FromMesh(a);
    const bCSG = babylonjs_1.CSG.FromMesh(b);
    const cCSG = babylonjs_1.CSG.FromMesh(c);
    // Set up a MultiMaterial
    const mat0 = new babylonjs_1.StandardMaterial('mat0', scene);
    const mat1 = new babylonjs_1.StandardMaterial('mat1', scene);
    mat0.diffuseColor.copyFromFloats(0.8, 0.2, 0.2);
    mat0.backFaceCulling = false;
    mat1.diffuseColor.copyFromFloats(0.2, 0.8, 0.2);
    mat1.backFaceCulling = false;
    let subCSG = bCSG.subtract(aCSG);
    let newMesh = subCSG.toMesh('csg', mat0, scene);
    newMesh.position = new babylonjs_1.Vector3(-10, 0, 0);
    subCSG = aCSG.subtract(bCSG);
    newMesh = subCSG.toMesh('csg2', mat0, scene);
    newMesh.position = new babylonjs_1.Vector3(10, 0, 0);
    subCSG = aCSG.intersect(bCSG);
    newMesh = subCSG.toMesh('csg3', mat0, scene);
    newMesh.position = new babylonjs_1.Vector3(0, 0, 10);
    // Submeshes are built in order : mat0 will be for the first cube, and mat1 for the second
    const multiMat = new babylonjs_1.MultiMaterial('multiMat', scene);
    multiMat.subMaterials.push(mat0, mat1);
    // Last parameter to true means you want to build 1 subMesh for each mesh involved
    subCSG = bCSG.subtract(cCSG);
    newMesh = subCSG.toMesh('csg4', multiMat, scene, true);
    newMesh.position = new babylonjs_1.Vector3(0, 0, -10);
    return scene;
}
exports.csgDemo = csgDemo;
//# sourceMappingURL=csg.js.map