"use strict";
// adapted from DEV article by Nick Scialli.
Object.defineProperty(exports, "__esModule", { value: true });
exports.testRedux = exports.createStore = void 0;
function createStore(reducer, initialState) {
    // real Redux has a third parameter, 'enhancer'
    let state = initialState;
    const listeners = [];
    function getState() {
        return state;
    }
    function dispatch(action) {
        state = reducer(state, action); // pass in state + action, get back new state
        listeners.forEach((listener) => listener(state));
    }
    function subscribe(fn) {
        listeners.push(fn);
        return () => listeners.splice(listeners.indexOf(fn), 1);
    }
    return { getState, dispatch, subscribe };
}
exports.createStore = createStore;
/// //////////////////////////////////
function testReducer(state, action) {
    switch (action.type) {
        case 'SET_USER_NAME':
            return Object.assign(Object.assign({}, state), { userName: action.payload // overwrite with provided payload
             });
        case 'SET_DISPLAY_MODE':
            return Object.assign(Object.assign({}, state), { displayMode: action.payload });
        default:
            return state; // not a payload we know?  just return state unmodified
    }
}
function testRedux() {
    // create a new store !!  using the testReducer function above.
    // most reducers will be much more complicated, and the objects
    // they store will be more complicated too
    const initialState = { userName: 'Guest', displayMode: 'light' };
    const store = createStore(testReducer, initialState);
    // change our user's name to 'Frankie'
    store.dispatch({ type: 'SET_USER_NAME', payload: 'Frankie' });
    // console.log(store.getState()) // {userName: "Frankie", displayMode: "light"}
    store.dispatch({ type: 'SET_DISPLAY_MODE', payload: 'dark' });
    // console.log(store.getState())  // {userName: "Frankie", displayMode: "dark"}
    console.assert(store.getState().displayMode == 'dark', store.getState());
}
exports.testRedux = testRedux;
//# sourceMappingURL=redux.js.map