"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Unittests = void 0;
const utilities_1 = require("./utilities");
/// /////////////////////////////////////////////
/// //// unit test this class
const consoleRed = 'background-color:red;color:white;';
const consoleYellow = 'background-color:yellow;color:black;';
const consoleBlue = 'background-color:blue;color:white;';
const consoleAqua = 'background-color:aqua;color:red;';
// this module is removed from PRODUCTION by the tree-shaker.  
// it is only available in DEVELOPMENT
class Unittests {
    constructor(app) {
        this.app = app;
    }
    async unittestAll() {
        console.log('%cStarting Unittests ' + this.app.version(), consoleAqua);
        this.allBabyMethodsTestor(this.app);
        // some non-visual tests
        this.testHTMLColor();
        // lets see the basic shapes, just a visual test
        let a = this.arrayOfThings(this.app);
        // let promise1 = new Promise((resolve, reject) => {
        //     this.move(a)
        // })
        // let promise2 = new Promise((resolve, reject) => {
        //     this.spin(a)
        // })
        this.spiral();
        this.move(a);
        await a[0].isIdle();
        this.spin(a);
        await a[0].isIdle();
        this.scale(a);
        await a[0].isIdle();
        this.dispose(a);
        // // now test some of the math
        // await this.getPosition(app)
        // await this.testPhysics(app)
        console.log('%cEnding Unittests', consoleAqua);
    }
    async allBabyMethodsTestor(app) {
        // test ALL Baby methods
        let methods = getAllMethodNames(app); // Baby
        // TODO: test for these, and for camera methods too
        // test ALL BabyThings methods
        let bt = app.shape('cube'); // create a BabyShape
        let btMethods = getAllMethodNames(bt); // get its methods (some may be BabyThings)
        // no test cases yet
        bt.dispose(); // don't need it anymore
        // console.log('btMethods', btMethods)
        for (let i = 0; i < btMethods.length; i++) { // use for() because can't async in codeblock  
            this.babyShapeTester(app, btMethods[i]); // and fire off each one
        }
    }
    async spiral() {
        let cube = this.app.cube();
        for (let i = 0; i < 5; i++) {
            let colors1 = ['red', 'yellow', 'green', 'blue', 'gold'];
            let colors2 = ['cyan', 'aqua', 'skyblue', 'purple', 'teal'];
            // await cube.penDown().color(colors[i]).moveAndTurn('left',5,'left',1).move('up',1).done()
            await cube.penDown().color(colors1[i]).move('left', 5).move('up', .5).done();
            await cube.penDown().color(colors2[i]).move('right', 5).move('up', .5).opacity(((4 - i) / 4)).done();
        }
        await cube.isIdle();
        cube.dispose();
    }
    /** a simple test against every method in BabyShape */
    async babyShapeTester(app, method) {
        switch (method) {
            case "constructor":
            case "onCollide":
            case "onClick":
            case "collide":
            case "addAxes":
            case "done":
            case "isIdle":
            case "dispose":
                break; // no test yet
            case "move":
                let btMove = app.shape('cube'); // start each test with a new box
                await btMove.move('forward', 5).move('left', 5).move('up', 5).done();
                let pMove = btMove.getPosition();
                console.assert(almostEqual(pMove.x, 5) && almostEqual(pMove.y, 5) && almostEqual(pMove.z, 5), `BabyShape.move [${pMove.z},${pMove.y},${pMove.z}]`);
                btMove.dispose(); // and get rid of it
                break;
            case "turn":
                let btTurn = app.shape('cube').addAxes(); // start each test with a new box
                await btTurn.turn('left', .25).done();
                await btTurn.turn('tiltLeft', .25).done();
                await btTurn.turn('forward', .25).done();
                await btTurn.turn('tiltRight', .25).done(); // should be back where we started
                let pTurn = btTurn.getRotation();
                console.assert(almostEqual(pTurn.x, 0) && almostEqual(pTurn.y, 0) && almostEqual(pTurn.z, 0), `BabyShape.turn [${pTurn.z},${pTurn.y},${pTurn.z}]`);
                btTurn.dispose(); // and get rid of it
                break;
            case "moveAndTurn":
            case "moveToward":
            case "moveToZero":
            case "turnToSame":
            case "turnToZero":
            case "size":
            case "color":
            case "opacity":
            case "addSkeleton":
            case "attachTo":
            case "break":
            case "setPosition":
            case "addToPosition":
            case "setRotation":
            case "addToRotation":
            case "getDistanceTo":
            case "constructor":
            case "getPosition":
            case "getRotation":
            case "getScale":
            case "hasOwnProperty":
            case "isPrototypeOf":
            case "propertyIsEnumerable":
            case "toString":
            case "valueOf":
            case "toLocaleString":
                break;
            default:
                console.assert(false, `No test provided for BabyShape.${method}`);
        }
    }
    async testAllBaby(app) {
        for (var member in app) { // For each member of the dictionary
            console.log('found a member ', member, typeof app[member]);
            if (typeof app[member] == "function") { // Is it a function?
                if (app.hasOwnProperty(member)) { // Not inherited
                    console.log('found member ', member); // do something...
                }
            }
        }
        // verifyAllFunctionsHandled(app1)
        // let app2 = app1.shape('cube')
        // verifyAllFunctionsHandled(app2)
        // // verify that all functions accounted for
        // function verifyAllFunctionsHandled(app) {
        // }
    }
    /** creates an array of things to test */
    arrayOfThings(app) {
        let ret = [];
        let x = -6;
        let y = 3; // up in the air
        let z = 5; // and a little back
        ret.push(app.shape('cube').setPosition(x += 2, y, z).addAxes().color('brown'));
        ret.push(app.shape('sphere').setPosition(x += 2, y, z).addAxes().color('cyan'));
        ret.push(app.shape('cylinder').setPosition(x += 2, y, z).addAxes().color('yellow'));
        ret.push(app.shape('cone').setPosition(x += 2, y, z).addAxes().color('blue'));
        ret.push(app.shape('capsule').setPosition(x += 2, y, z).addAxes().color('yellow'));
        ret.push(app.shape('torus').setPosition(x += 2, y, z).addAxes().color('blue'));
        return ret; // return one of each...
    }
    /** spin the collection of things we were passed */
    async spin(a) {
        a.forEach(async (i) => {
            await i.turn('left', .25).done();
            await i.turn('tiltLeft', .25).done();
            await i.turn('forward', .25).done();
            await i.turn('tiltRight', .25).done(); // should be back where we started
        });
    }
    /** just a jump to the left, and then a move to the right... */
    async move(a) {
        a.forEach(async (i) => {
            await i.move('left', 2).done();
            await i.move('forward', 2).done();
            await i.move('right', 2).done();
            await i.move('backward', 2).done();
        });
    }
    /** just a jump to the left, and then a move to the right... */
    async scale(a) {
        a.forEach(async (i) => {
            await i.size(2).done();
            await i.size(.2).done();
            await i.size(1).done();
        });
    }
    /** dispose of an array of shapes */
    async dispose(a) {
        a.forEach(async (i) => {
            i.dispose();
        });
    }
    // gentle test of getPosition(), move(), getDistanceTo(), moveToward()
    async getPosition(app) {
        const aTest = app.shape('cube').color('blue').addAxes();
        await aTest.move('left', 3).move('backward', 4).done();
        let position = aTest.getPosition();
        console.assert(almostEqual(position.x, -3), `${position.x} should be -3`);
        // by pythagoras , the third side should be size '5'
        console.assert(almostEqual(aTest.getPosition().length(), 5), ` ${aTest.getPosition().length()}should be 5`);
        const bTest = app.shape('cube').color('blue').addAxes();
        let distance = bTest.getDistanceTo(aTest);
        console.assert(almostEqual(distance, 5), ` ${distance}should be 5`);
        await bTest.moveToward(aTest, distance).done();
        let distance2 = bTest.getDistanceTo(aTest);
        console.assert(almostEqual(distance2, 0), ` ${distance2}should be 0`);
        aTest.dispose();
        bTest.dispose();
    }
    async testPhysics(app) {
        const balls = [];
        balls.push(app.shape('sphere').collide('active').move('left', 0.2, 0.1).move('forward', 0.2, 0.1));
        for (let i = 0; i < 30; i++) {
            balls.push(app.shape('sphere').collide('active').move('up', 2 + 1.1 * i, 0));
        }
    }
    testHTMLColor() {
        const c = utilities_1.HTMLColor('blue'); // color3 values are between 0 and 1
        console.assert(c.r == 0, c.toString());
        console.assert(c.g == 0, c.toString());
        console.assert(c.b == 1, c.toString());
        let t = utilities_1.HTMLColor('Yellow').toHexString();
        console.assert(t == '#FFFF00', `Yellow isn't '${t}'`);
        t = utilities_1.HTMLColor('White').toHexString();
        console.assert(t == '#FFFFFF', `White isn't '${t}'`);
        // test lowercase 'aliceblue
        t = utilities_1.HTMLColor('AliceBlue').toHexString();
        console.assert(t == '#F0F8FF', `aliceblue isn't '${t}'`);
    }
}
exports.Unittests = Unittests;
// lots of rounding errors creep in, we sometimes need to check 'almost right'
function almostEqual(a, b) {
    let maxDiff = Math.max(Math.abs(a), Math.abs(b), 0.5) / 100; // 1% of the bigger value (plus guard for div-by-zero)
    return (Math.abs(a) - Math.abs(b) < Math.abs(maxDiff)); // error is less than 1% of bigger value 
}
/////////////////////////////////////////
/////////////////////////////////////////
/////////////////////////////////////////
async function testMove_Turn_Move(app) {
    console.log(app);
    const aTest = app.shape('cube').color('blue').addAxes();
    for (let i = 1; i < 5; i++) {
        await aTest.move('forward', 1).done();
        await aTest.turn('left', 0.25).done();
    }
}
function testCollide(app) {
    const b1 = app.shape('sphere');
    const b2 = app.shape('sphere').onCollide((who) => {
        if (who == b1) {
            // b1 and b2 collided
        }
    });
}
async function testPlace(app) {
    const duration = 3; // set longer if you want to see this test
    const aTest = app.shape('cube').addAxes();
    await aTest.color('green', duration).done();
    await aTest.color('pink', duration).done();
    await aTest.color('red', duration).done();
    await aTest.color('blue', duration).done();
    await aTest.color('green', duration).done();
    await aTest.color('pink', duration).done();
}
// async function testMove (app:Baby) {
//   const duration = 1 // set longer if you want to see this test
//   // console.log('testMove')
//   const aTest = app.shape('cube')
//   // move it up 5, see what happens
//   const position1 = aTest.baby.babylonMeshs.shapeMap.get(aTest.uuid).babylonMesh.position.clone()
//   // console.log('position1',position1)
//   await aTest.move('up', 5, duration).done()
//   const position2 = aTest.baby.babylonMeshs.shapeMap.get(aTest.uuid).babylonMesh.position.clone()
//   // console.log('position2',position2)
//   console.assert(Math.abs(position2.y - position1.y - 5) < 0.1, `2: abs(${position2.y} - ${position1.y} - 5) should be close to 0`)
//   // move it right, see what happens
//   await aTest.move('right', 5, duration).done()
//   const position3 = aTest.baby.babylonMeshs.shapeMap.get(aTest.uuid).babylonMesh.position.clone()
//   // console.log('position3',position3)
//   console.assert(Math.abs(position3.x - position1.x - 5) < 0.1, `3: abs(${position3.x} - ${position1.x} - 5) should be close to 0`)
//   // move it down, see what happens
//   await aTest.move('down', 5, duration).done()
//   const position4 = aTest.baby.babylonMeshs.shapeMap.get(aTest.uuid).babylonMesh.position.clone()
//   // console.log('position4',position4)
//   console.assert(Math.abs(position4.y - position1.y) < 0.1, `4: abs(${position4.x} - ${position1.x}) should be close to 0`)
//   // move it backward, see what happens
//   await aTest.move('backward', 5, duration).done()
//   const position5 = aTest.baby.babylonMeshs.shapeMap.get(aTest.uuid).babylonMesh.position.clone()
//   console.assert(Math.abs(position5.z - position1.z - 5) < 0.1, `5: abs(${position5.x} - ${position1.x} -5) should be close to 5`)
//   // two moves, diagonal back to home
//   aTest.move('left', 5, duration) // in parallel with next
//   await aTest.move('forward', 5, duration).done()
//   const position6 = aTest.baby.babylonMeshs.shapeMap.get(aTest.uuid).babylonMesh.position.clone()
//   console.assert(Math.abs(position6.x - position1.x) < 0.1, `6: abs(${position6.x} - ${position1.x} ) should be close to 5`)
//   aTest.dispose()
// }
// async function testTurn (app:Baby) {
//   const duration = 1 // set longer if you want to see this test
//   const angle = 0.5
//   const aTest = app.shape('cube').addAxes()
//   await aTest.turn('left', angle, duration).done()
//   await aTest.turn('forward', angle, duration).done()
//   await aTest.turn('tiltLeft', angle, duration).done()
//   await aTest.turn('right', angle, duration).done()
//   await aTest.turn('backward', angle, duration).done()
//   await aTest.turn('tiltRight', angle, duration).done()
//   const rotation = aTest.baby.babylonMeshs.shapeMap.get(aTest.uuid).babylonMesh.rotation
//   console.assert(Math.abs(rotation.x) + Math.abs(rotation.y) + Math.abs(rotation.z) < 0.1, 'rotation should be close to 05')
//   aTest.dispose()
// }
async function testMoveTurn(app) {
    const duration = 3; // set longer if you want to see this test
    const aTest = app.shape('cube').addAxes();
    // await aTest.move('forward',0,3).done()
    // await aTest.move('forward',10,duration).turn('left',1,duration).done()
    await aTest.moveAndTurn('forward', 10, 'left', 0.33, duration).done();
    await aTest.moveAndTurn('forward', 10, 'left', 0.33, duration).done();
    await aTest.moveAndTurn('forward', 10, 'left', 0.33, duration).done();
    await aTest.moveAndTurn('forward', 25, 'backward', 1, duration).done();
    await aTest.moveAndTurn('right', 25, 'tiltLeft', 1, duration).done();
    // aTest.dispose()
}
async function testMoveToward(app) {
    const duration = 3; // set longer if you want to see this test
    const aTest = app.shape('cube').addAxes();
    const bTest = app.shape('sphere').addAxes();
    for (let i = 0; i < 10; i++) {
        aTest.color('blue', duration);
        await aTest.moveAndTurn('forward', 5, 'left', 0.25, duration).done();
        aTest.color('red', duration);
        await bTest.moveToward(aTest, 5, duration).done();
    }
}
function getAllMethodNames(obj) {
    let methods = [];
    while (obj = Reflect.getPrototypeOf(obj)) {
        let keys = Reflect.ownKeys(obj);
        keys.forEach((k) => {
            let sK = k.toString(); // keys are not always strings
            if (!sK.includes('_')) // take out the ones like __getter
                methods.push(sK);
        });
    }
    return methods;
}
//# sourceMappingURL=unittest.js.map