"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
// start by importing everything
const baby_1 = require("../frontend/baby");
// define the size of the board
const xMax = 20; // best with even numbers, or half-squares at sides of floor
const yMax = 20;
// initialize the game engine.
const app = new baby_1.Baby().keydown(snakeKeyDown); //.inspector()
// define the 'global' values outside all functions so available to all the functions
let snakeHead; // pointer to current snakehead
const snake = []; // array of snake segments
const snakeMoves = []; // the Nth segment should move in the Nth direction of this array
const camera = app.camera().moveToZero(5); //.move('backward',20,0)
const floor = app.floor(xMax, yMax, 5); //  majorgrid lines every 5
const apple = app.shape('sphere').color('green').move('left', 3, 0); // an apple that we place randomly
const duration = 0.2;
// these are the bumpers
let top = app.shape('cube');
let bottom = app.shape('cube');
let left = app.shape('cube');
let right = app.shape('cube');
let direction = 'forward';
app.GUI('button').position("Top").text('Ready Player One');
app.GUI('button').position("TopLeft").text('Use the WASD keys').role('Danger');
// we want the game in an ASYNC function.  
snakeGame(app); // and run the game in an async function
async function snakeGame(app) {
    // set up 'teleport' bumpers at the edges of the board
    // should be in zero time, but default to 1 second for tutorial purposes
    top.move('backward', yMax / 2 + 0.5).size([xMax, 1, 1]).opacity(.1).collide('solid');
    bottom.move('forward', yMax / 2 + 0.5).size([xMax, 1, 1]).opacity(.1).collide('solid');
    left.move('left', xMax / 2 + 0.5).size([1, 1, yMax]).opacity(.1).collide('solid');
    right.move('right', xMax / 2 + 0.5).size([1, 1, yMax]).opacity(.1).collide('solid');
    await top.done(); // could use ANY bumper, they all set up at the same speed
    apple.collide('solid').onCollide((other) => {
        if (other == snakeHead) {
            // because the snake hit us, add a new element to the front of the snake at the
            // position of the apple
            let newElement = newSnakeElement();
            newElement.moveToward(apple, newElement.getDistanceTo(apple), 0);
            snake.unshift(newElement); // add a new element at the FRONT of the snake 
            snakeHead = newElement; // in JS, this is just a pointer
            snakeHead.move(direction, 1, duration); // move and wait
            snakeHead.color('red');
            // move it forward/backward and left/right by some random amount, in zero time
            apple.moveToZero(0);
            apple.move('forward', app.randomIntBetween(-yMax / 2 + 1, yMax / 2), 0);
            apple.move('left', app.randomIntBetween(-xMax / 2 + 1, xMax / 2), 0);
        }
    });
    // add the first element to the snake array
    snakeHead = newSnakeElement();
    snake.push(snakeHead);
    snakeMoves.push('forward');
    // start the action.  from here, it's all driven by collisions and keystrokes
    for (;;) { // while (true) {
        for (let i = 1; i < snake.length; i++)
            snake[i].move(snakeMoves[i], 1, duration);
        // console.log(snakeMoves)
        await snakeHead.move(direction, 1, duration).done(); // move and wait
        snakeMoves.unshift(direction); // add direction to the top of the move queue
        if (snakeMoves.length > snake.length)
            snakeMoves.pop(); // don't need these at the end
    }
}
function newSnakeElement() {
    // define a snake element with a collision function.
    let snakeElement = app.shape('cube');
    snakeElement.color('red', 0); // take a second to color to red
    // add a collision event to 'teleport' if we hit a bumper
    snakeElement.collide('solid').onCollide((other) => {
        switch (other) {
            case top:
                snakeElement.move('forward', yMax - 1, 0);
                break;
            case bottom:
                snakeElement.move('backward', yMax - 1, 0);
                break;
            case left:
                snakeElement.move('right', xMax - 1, 0);
                break;
            case right:
                snakeElement.move('left', xMax - 1, 0);
                break;
            default:
            // throw ('did not expect to collide with something else')
        }
    });
    snakeMoves.unshift(direction); // add direction to the top of the move queue for this cube
    return (snakeElement);
}
// ask for keyboard notifications
function snakeKeyDown(data) {
    console.log('in snakekeyDown');
    switch (data['code']) {
        case 'KeyA':
            direction = 'left';
            break;
        case 'KeyW':
            direction = 'forward';
            break;
        case 'KeyD':
            direction = 'right';
            break;
        case 'KeyS':
            direction = 'backward';
            break;
        default:
    }
}
//# sourceMappingURL=snake.js.map