"use strict";
/* eslint-disable @typescript-eslint/ban-types */ // allow 'Function' as a type
Object.defineProperty(exports, "__esModule", { value: true });
exports.ColorTween = exports.ScalarTween = exports.V3Tween = exports.MiscTween = exports.NullTween = exports.Tween = void 0;
// origial Copyright (c) 2012 Ash Blue (blueashes.com)
// downloaded from github, converted to TS, and heavily modified by Tom Berend
/// /////////////////////////////////////////////////////////////////////////////////////////
const babylonjs_1 = require("babylonjs");
// import { HTMLColor } from './htmlcolor'
// code for testing
// let a = app.shape('cube').addToPosition([-8, 4, 0]).setColor('red')
// let b = app.shape('cube').addToPosition([-8, 2, 0]).setColor('green')
// let c = app.shape('cube').addToPosition([-8, 0, 0]).setColor('black')
// let d = app.shape('cube').addToPosition([-8, -2, 0])
// let e = app.shape('cube').addToPosition([-8, -4, 0])
// let ta = new Tween(-8, 16, 10 , "Begin And End Fast",(value)=>{a.setPosition([value, 4, 0])})
// let tb = new Tween(-8, 16, 10 , "Begin And End Normally",(value)=>{b.setPosition([value, 2, 0])})
// let tc = new Tween(-8, 16, 10 , "Begin And End Slow",(value)=>{c.setPosition([value, 0, 0])})
// let td = new Tween(-8, 16, 10 , "Begin Fast, End Slow",(value)=>{d.setPosition([value, -2, 0])})
// let te = new Tween(-8, 16, 10 , "Begin Slow, End Fast",(value)=>{e.setPosition([value, -4, 0])})
// app.update = function () {
//     ta.doTween()
//     tb.doTween()
//     tc.doTween()
//     td.doTween()
//     te.doTween()
// }
//
const consoleGold = 'background-color:gold:color:white;';
const consoleRed = 'background-color:red;color:white;';
const consoleYellow = 'background-color:yellow;color:black;';
const consoleBlue = 'background-color:blue;color:white;';
/**
 * Supports easing for the following commands you can demo at
 * http://ashblue.github.com/canvas-tween-demo/ 'linear', 'quadIn', 'quadOut',
 * 'quadInOut', 'cubeIn', 'cubeOut', 'cubeInOut', 'quartIn', 'quartOut', 'quartInOut',
 * 'quintIn', 'quintOut', 'quintInOut', 'sineIn', 'sineOut', 'sineInOut', 'expoIn',
 * 'expoOut', 'expoInOut', 'circIn', 'circOut', 'circInOut'. Adopted from
 * http://gizma.com/easing/
 * @link http://ashblue.github.com/canvas-tween-demo/
 */
class EasingLibrary {
    /**
       * @param {number} t Current time in millseconds
       * @param {number} b Start value
       * @param {number} c Distance traveled relative to the start value
       * @param {number} d Duration in milliseconds
       */
    static linear(t, b, c, d) {
        // console.log(`linear easing t=${t},b=${b},c=${c},d=${d}`)
        t /= d;
        return c * t + b;
    }
    static quadIn(t, b, c, d) {
        t /= d;
        return c * t * t + b;
    }
    static quadOut(t, b, c, d) {
        t /= d;
        return -c * t * (t - 2) + b;
    }
    static quadInOut(t, b, c, d) {
        t /= d / 2;
        if (t < 1) {
            return c / 2 * t * t + b;
        }
        t--;
        return -c / 2 * (t * (t - 2) - 1) + b;
    }
    // static quadOutIn(t, b, c, d) {   // new, for Baby  fast in, slow middle fast out, slow in middle
    //     t /= d / 2;
    //     if (t < 1) {
    //         return c / 2 * t * t + b;
    //     }
    //     t--;
    //     return -c / 2 * (t * (t - 2) - 1) + b;
    // }
    // t is (currentTime - startTime) = how long we have been running, say 3 seconds
    // d is how long we should run, say 10 seconds
    //  so d/2 is the halfway time and t /= d/2 is either less than or greater than 1
    // b is the start value,
    // c is the add travel
    static quadOutIn(t, b, c, d) {
        t /= d / 2;
        if (t < 1) {
            // in the first half we want to slow down
            return (-c / 2) * (t * (t - 2)) + b;
        }
        // in the second half we want to speed up
        t--;
        return (c / 2) + (c / 2 * t * t) + b;
    }
    static cubeIn(t, b, c, d) {
        t /= d;
        return c * t * t * t + b;
    }
    static cubeOut(t, b, c, d) {
        t /= d;
        t--;
        return c * (t * t * t + 1) + b;
    }
    static cubeInOut(t, b, c, d) {
        t /= d / 2;
        if (t < 1) {
            return c / 2 * t * t * t + b;
        }
        t -= 2;
        return c / 2 * (t * t * t + 2) + b;
    }
    static quartIn(t, b, c, d) {
        t /= d;
        return c * t * t * t * t + b;
    }
    static quartOut(t, b, c, d) {
        t /= d;
        t--;
        return -c * (t * t * t * t - 1) + b;
    }
    static quartInOut(t, b, c, d) {
        t /= d / 2;
        if (t < 1) {
            return c / 2 * t * t * t * t + b;
        }
        t -= 2;
        return -c / 2 * (t * t * t * t - 2) + b;
    }
    static quintIn(t, b, c, d) {
        t /= d;
        return c * t * t * t * t * t + b;
    }
    static quintOut(t, b, c, d) {
        t /= d;
        t--;
        return c * (t * t * t * t * t + 1) + b;
    }
    static quintInOut(t, b, c, d) {
        t /= d / 2;
        if (t < 1) {
            return c / 2 * t * t * t * t * t + b;
        }
        t -= 2;
        return c / 2 * (t * t * t * t * t + 2) + b;
    }
    static sineIn(t, b, c, d) {
        return -c * Math.cos(t / d * (Math.PI / 2)) + c + b;
    }
    static sineOut(t, b, c, d) {
        return c * Math.sin(t / d * (Math.PI / 2)) + b;
    }
    static sineInOut(t, b, c, d) {
        return -c / 2 * (Math.cos(Math.PI * t / d) - 1) + b;
    }
    static expoIn(t, b, c, d) {
        return c * Math.pow(2, 10 * (t / d - 1)) + b;
    }
    static expoOut(t, b, c, d) {
        return c * (-Math.pow(2, -10 * t / d) + 1) + b;
    }
    static expoInOut(t, b, c, d) {
        t /= d / 2;
        if (t < 1) {
            return c / 2 * Math.pow(2, 10 * (t - 1)) + b;
        }
        t--;
        return c / 2 * (-Math.pow(2, -10 * t) + 2) + b;
    }
    static circIn(t, b, c, d) {
        t /= d;
        return -c * (Math.sqrt(1 - t * t) - 1) + b;
    }
    static circOut(t, b, c, d) {
        t /= d;
        t--;
        return c * Math.sqrt(1 - t * t) + b;
    }
    static circInOut(t, b, c, d) {
        t /= d / 2;
        if (t < 1) {
            return -c / 2 * (Math.sqrt(1 - t * t) - 1) + b;
        }
        t -= 2;
        return c / 2 * (Math.sqrt(1 - t * t) + 1) + b;
    }
}
// a tween is really a promise, maybe should just subclass 'Promise' ?
/**
 * Constructor for the tween
 * @param {number} startValue What value does the tween start at
 * @param {number} distance How far does the tween's value advance from the startValue?
 * @param {number} durationMS Amount of time in milliseconds the tween runs for
 * @param {Function} animate What easing function should be used from the easing library?
 * See EasingLibrary for a list of potential easing equations.
 * @param {string} loop Can be left blank, set to loop, or repeat. Loop repeats repeats the animation
 * in reverse every time. Repeat will run the original tween from the beginning
 * @returns {self}
 */
class Tween {
    constructor(name = '') {
        this.isCancelled = false;
        this.tweenType = null; // need to know for some operations
        /**
         * Has the tween expired yet?
         * @returns {boolean} True if the tween has expired
         */
        this.expired = () => { return true; }; // we override this method, so code it to look like a property
        this.name = name; // just for debugging
        this.promise = new Promise((resolve) => {
            this.promiseResolve = resolve;
        });
    }
    commonStart(timeStamp, seconds, animate, updateFunction) {
        this.startTime = Date.now();
        this.timeStamp = timeStamp;
        this.durationMS = seconds * 1000; // internally we use ms
        this.updateFunction = updateFunction;
        this.expired = () => { return this.startTime + this.durationMS < Date.now(); };
        // console.log(animate)
        switch (animate) {
            case '1-Begin And End Normally':
                this.animationFunction = (t, b, c, d) => (EasingLibrary.linear(t, b, c, d));
                break;
            case '2-Begin Slow, End Fast':
                this.animationFunction = (t, b, c, d) => (EasingLibrary.quadIn(t, b, c, d));
                break;
            case '3-Begin Fast, End Slow':
                this.animationFunction = (t, b, c, d) => (EasingLibrary.quadOut(t, b, c, d));
                break;
            case '4-Begin And End Fast':
                this.animationFunction = (t, b, c, d) => (EasingLibrary.quadOutIn(t, b, c, d));
                break;
            case '5-Begin And End Slow':
                this.animationFunction = (t, b, c, d) => (EasingLibrary.quadInOut(t, b, c, d));
                break;
        }
        return this;
    }
    /**
     * Rounds the passed number to two decimal places. Prevents large float
     * numbers from being multiplied
     * @param {number} num number you want to round
     * @returns {number} Rounded number
     */
    round(num) {
        // TODO use 'this.minimum' as the round factor
        return Math.round(num * 1000) / 1000;
    }
    // /**
    //  * Get the current value of the tween
    //  * @param {number} dateNow is 'Date.now' from the caller (more accurate)
    //  * @returns {number} Current value of the tween
    //  */
    // getValue(dateNow: number): number {
    //     // Run normally
    //     let total: number
    //     if (!this.expired()) {
    //         total = this.animationFunction(dateNow - this.startTime, this.startValue, this.distance, this.duration);
    //     } else {            // Ended and no repeat is present   --- expired
    //         total = this.startValue + this.distance;
    //     }
    //     return this.round(total);
    // };
    /**
     * Retrieves the start time relative to the time passed from the previous start time
     * @returns {number} Start time of the tween relative to time passed
     */
    getStartTime() {
        return Date.now() - this.startTime - this.durationMS + Date.now();
    }
    /**
     * Resets the tween and runs it relative to the current time
     * @returns {self}
     */
    reset() {
        this.startTime = Date.now();
        return this;
    }
}
exports.Tween = Tween;
// some things like shape creation doesn't need a tween
class NullTween extends Tween {
    constructor(name) { super(name); }
    doTween() { return (this); }
}
exports.NullTween = NullTween;
////////////////////////////////////////////
// here are the actual tweens we support ///
//  V3Tween()      // process a Vector3
//  ScalarTween()  // process a scalar value
//  MiscTween()    // just a promise that resolves when the shape has no other tweens
////////////////////////////////////////////
/** a promise that resolves true when the update function returns true */
class MiscTween extends Tween {
    constructor(name) {
        super(name);
    }
    start(updateFunction) {
        this.tweenType = 'misc';
        this.startTime = Date.now();
        this.updateFunction = updateFunction;
        this.expired = () => { return this.updateFunction(); };
        return this;
    }
    doTween() {
        return this;
    }
}
exports.MiscTween = MiscTween;
class V3Tween extends Tween {
    constructor(name) {
        super(name);
    }
    start(tweenType, startValue, endValue, timeStamp, seconds, animate, updateFunction) {
        // console.log('%cV3 tween.start',consoleBlue, startValue,endValue)
        console.assert(typeof startValue !== 'undefined', 'missing startValue');
        console.assert(typeof startValue.x !== 'undefined', 'startValue not a Vector3');
        console.assert(typeof endValue !== 'undefined', 'missing endValue');
        console.assert(typeof endValue.x !== 'undefined', 'endValue not a Vector3 ');
        // some things are the same for every type of tween
        this.commonStart(timeStamp, seconds, animate, updateFunction);
        this.startValue = startValue;
        this.endValue = endValue;
        this.moveValue = this.endValue.subtract(this.startValue);
        this.tweenType = tweenType;
        this.minimum = 0.001; // the smallest move we will make
        this.lastDistance = startValue.clone(); // where we think the mesh is relative to start
        return this;
    }
    /**
  * Run the tween computation and update whatever it is supposed to
  * @param {number} dateNow is 'Date.now' from the caller (more accurate)
  * @returns {number} Current value of the tween
  */
    doTween() {
        // we have start(V3) and distance(V3).  but we can't just move between them, there might be two tweens
        // we send out a stream of delta-positions ( move +.023, move +.041...)
        // and we keep track of the last position we think the mesh is at
        //
        // further, we use a MINIMUM, if the minimum is .01 then we don't issue any moves less than that.
        // and we round down to that value.  might be a tiny bit jerky, but much less overhead
        //
        // for colors, the minimum is 1, because colors don't have fractions.
        // console.log('%cV3 tween.doTween',consoleBlue, this.startValue,this.endValue)
        console.assert(typeof this.startValue !== 'undefined', 'missing startValue');
        console.assert(typeof this.startValue.x !== 'undefined', 'startValue not a Vector3');
        console.assert(typeof this.endValue !== 'undefined', 'missing endValue');
        console.assert(typeof this.endValue.x !== 'undefined', 'endValue not a Vector3 ');
        let newDistance;
        const timeNow = Date.now() - this.startTime;
        // console.log(`Tweening '${this.name}'`)
        // we only want to move if the tween is still alive
        if (!this.isCancelled) {
            if (!this.expired()) { // still active, so delta movement
                // console.log(`timenow is`, timeNow, `startTime is `,this.startTime)
                // these three look complicated, but they are simply a call to the easing function
                const d0 = this.round(this.animationFunction(timeNow, this.startValue.x, this.moveValue.x, this.durationMS));
                const d1 = this.round(this.animationFunction(timeNow, this.startValue.y, this.moveValue.y, this.durationMS));
                const d2 = this.round(this.animationFunction(timeNow, this.startValue.z, this.moveValue.z, this.durationMS));
                newDistance = new babylonjs_1.Vector3(d0, d1, d2);
                // console.log(this.name,this.startValue,newDistance,'in getValue at time ',timeNow,this.durationMS)
            }
            else { // Ended and no repeat is present   --- expired
                newDistance = this.endValue.clone();
            }
            const newDelta = newDistance.subtract(this.lastDistance);
            this.lastDistance = newDistance;
            this.updateFunction(newDelta); // might move or turn, whatever
        }
        return this;
    }
}
exports.V3Tween = V3Tween;
class ScalarTween extends Tween {
    constructor(name) {
        super(name);
    }
    start(tweenType, startValue, endValue, timeStamp, seconds, animate, updateFunction) {
        // some things are the same for every type of tween
        this.commonStart(timeStamp, seconds, animate, updateFunction);
        this.startValue = startValue;
        this.endValue = endValue;
        this.moveValue = this.endValue - this.startValue;
        this.tweenType = tweenType;
        this.minimum = 0.01; // the smallest move we will make
        this.lastDistance = startValue; // where we think the mesh is relative to start
        // console.log(this.name, 'startvalue', this.startValue)
        // console.log('endValue', this.endValue)
        return this;
    }
    /**
  * Run the tween computation and update whatever it is supposed to
  * @param {number} dateNow is 'Date.now' from the caller (more accurate)
  * @returns {number} Current value of the tween
  */
    doTween() {
        // we are in scalarTween
        // for colors, the minimum is 1, because colors don't have fractions.
        let newDistance;
        const timeNow = Date.now() - this.startTime;
        // console.log(`Tweening '${this.name}'`)
        if (!this.expired()) {
            // console.log(`timenow is`, timeNow, `startTime is `,this.startTime)
            // these three look complicated, but they are simply a call to the easing function
            newDistance = this.round(this.animationFunction(timeNow, this.startValue, this.moveValue, this.durationMS));
            if (newDistance < this.minimum) // may be a bit jerky, but reduces accumulated error
             {
                newDistance = 0;
            }
            // console.log(this.startValue,this.moveValue,'in getValue at time ',timeNow,this.durationMS)
        }
        else { // Ended and no repeat is present   --- expired
            newDistance = this.endValue;
            // newDistance = this.startValue.clone().add(this.endValue)
        }
        const newDelta = newDistance - this.lastDistance;
        // console.log('--')
        // console.log('start', this.startValue)
        // console.log('moveValue',this.moveValue)
        // console.log('newDistance', newDistance)
        // console.log('newDelta', newDelta)
        this.lastDistance = newDistance;
        // console.log('delta',this.lastDistance,newDistance, newDelta.subtract(this.startValue))
        // console.log(`${this.name} V3Tween new delta ${newDelta.x},${newDelta.y},${newDelta.z}`)
        this.updateFunction(newDelta); // might move or turn, whatever
        return this;
    }
}
exports.ScalarTween = ScalarTween;
class ColorTween extends Tween {
    constructor(name) {
        super(name);
    }
    start(tweenType, startValue, distance, timeStamp, seconds, animate, updateFunction) {
        // some things are the same for every type of tween
        this.commonStart(timeStamp, seconds, animate, updateFunction);
        this.startValue = startValue;
        this.distance = distance;
        this.minimum = 1; // the smallest move we will make
        this.lastDistance = startValue; // where we think the mesh is
        this.tweenType = tweenType;
        return this;
    }
    /** Run the tween computation and update whatever it is supposed to */
    doTween() {
        // console.log('chocolate is ', HTMLColor('chocolate'))
        let newDistance;
        const timeNow = Date.now() - this.startTime;
        // console.log(`Tweening '${this.name}'`)
        if (!this.expired()) {
            // console.log(`timenow is`, timeNow, `startTime is `,this.startTime)
            // these three look complicated, but they are simply a call to the easing function
            const d0 = this.round(this.animationFunction(timeNow, this.startValue.r, this.distance.r, this.durationMS));
            const d1 = this.round(this.animationFunction(timeNow, this.startValue.g, this.distance.g, this.durationMS));
            const d2 = this.round(this.animationFunction(timeNow, this.startValue.b, this.distance.b, this.durationMS));
            newDistance = new babylonjs_1.Color3(d0, d1, d2);
            // console.log(this.startValue, this.distance, 'in getValue at time ', timeNow, this.durationMS)
        }
        else { // Ended and no repeat is present   --- expired
            newDistance = this.startValue.add(this.distance);
        }
        const newDelta = new babylonjs_1.Color3(newDistance.r - this.lastDistance.r, newDistance.g - this.lastDistance.g, newDistance.b - this.lastDistance.b);
        // console.log('start and new and delta' ,this.startValue,newDistance,newDelta)
        this.lastDistance = newDistance;
        // console.log('delta',this.lastDistance,newDistance, newDelta.subtract(this.startValue))
        // console.log(`${this.name} V3Tween new delta ${newDelta.x},${newDelta.y},${newDelta.z}`)
        this.updateFunction(newDelta); // might move or turn, whatever
        return this;
    }
}
exports.ColorTween = ColorTween;
// constructor(start: Vector3, end: Vector3, startTime:number,duration: number,
//     animate: "Begin And End Normally" | "Begin Slow, End Fast" | "Begin Fast, End Slow" | "Begin And End Fast" | "Begin And End Slow",
//         ) {
//     let dDistance = tNode.position.subtract(handle.babylonMesh.position)
//     let ta = new MoveTween(start, cmd.distance, cmd.duration, cmd.animate, (value) => { cmd.tweenCmd([value]) })
//     // TODO:       this.tweenArray.push(ta)
//     return this.newTweens
//# sourceMappingURL=tween.js.map