"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.VT52 = exports.World = void 0;
const babyengine_1 = require("../middle/babyengine");
/** This is the factory, use the concrete classes below (like VT52) */
class World {
    constructor() {
        this.canvas = document.getElementById('canvas');
        this.ctx = this.canvas.getContext('2d');
        this.canvas.height = Math.max(this.canvas.height, 1024);
        this.canvas.width = Math.max(this.canvas.width, 1280);
        this.height = this.canvas.height;
        this.width = this.canvas.width;
        console.log('canvas reports', this.height, this.width);
        this.observer = new babyengine_1.Observers();
    }
}
exports.World = World;
const pixelX = 14;
const pixelY = 20;
const rows = 24;
const cols = 40;
const vOffset = 10; // correct for characters (not for squares)
/** An old-fashioned cathod-ray-terminal (CRT), like the legendary DEC VT52 */
class VT52 extends World {
    constructor() {
        super();
        this.cursorX = 0;
        this.cursorY = 0;
        this.pressedKeyBuffer = [];
        // // width should always be 640.  640 / 80 is 8 pixels across, which defines the character size
        // console.assert(this.width >= 640, 'expect VT52 canvas to be 640 px wide')
        // console.assert(this.height >= 480, 'expect VT52 canvas to be 480 px high')
        this.displayBuffer = Array(rows * cols).fill(0);
        babyengine_1.Observers.addObserver('keydown', (data) => {
            this.processVT52Key(data);
        });
        this.testVT52();
    }
    testVT52() {
        this.drawScreen();
        this.printString('now is the time for all good men to come to the aid of the party.');
        this.printString('\nnow is the time for all good men to come to the aid of the party.\n\nWalla Walla Washington.');
        this.drawScreen();
        for (let i = 0; i < 20; i++) {
            this.printString(`\ncounting ${i}`);
        }
        this.drawScreen();
        // // subscript to mouse and keyboard events
        // this.mouseObservable.addObserver('mousedown', this.mouseSoftKey, this)
        // this.animationObservable.addObserver('tick', this.animate, this)
        // // and publish when we have a new key (mouse or kybd)
        // this.answerReceived = new Observable()
    }
    processVT52Key(event) {
        let pressed = event.key.charCodeAt(0); // convert to number
        this.pressedKeyBuffer.push(pressed);
        this.printChar(pressed);
        this.drawScreen();
    }
    // full refresh of screen.
    drawScreen() {
        this.ctx.font = "12px Courier";
        this.ctx.clearRect(0, 0, this.canvas.width, this.canvas.height);
        for (let j = 0; j < rows; j++) {
            for (let i = 0; i < cols; i++) {
                let charCode = this.displayBuffer[j * cols + i];
                if (charCode === 0) {
                    // a null shows as a small blue rectangle
                    this.ctx.fillStyle = 'blue';
                    this.ctx.fillRect(i * pixelX + 4, j * pixelY + 6, 2, 2);
                }
                else {
                    // console.log('charCode', j, i, charCode)
                    let charString = String.fromCharCode(charCode);
                    this.ctx.fillStyle = "black";
                    this.ctx.fillText(charString, i * pixelX, j * pixelY + vOffset);
                }
            }
        }
    }
    printString(chars) {
        chars.split('').forEach((char) => this.printChar(char.charCodeAt(0)));
    }
    printChar(charCode) {
        console.assert(this.cursorX < cols, `cursorX should always be less than ${cols}`);
        if (charCode === 10) { // force CRLF
            this.printCRLF();
        }
        else {
            this.displayBuffer[cols * this.cursorY + this.cursorX] = charCode;
            this.cursorX += 1; // horizontal cursor position
            if (this.cursorX >= cols) {
                this.printCRLF();
            }
        }
    }
    printCRLF() {
        console.log(`printCRLF at ${this.cursorX},${this.cursorY}`);
        if (this.cursorY === (rows - 1)) {
            // we are on the bottom line, so scroll up
            for (let i = cols; i < this.displayBuffer.length; i++) {
                this.displayBuffer[i - cols] = this.displayBuffer[i];
                this.displayBuffer[i] = 0;
            }
            this.cursorX = 0; // cursorY remains the bottom row
        }
        else {
            // just drop the cursor to the next line
            this.cursorY += 1;
            this.cursorX = 0;
        }
    }
}
exports.VT52 = VT52;
//# sourceMappingURL=vt52.js.map